package jwiley.chp10;
/**
 *  ComponentHolder.java
 *  A class that demonstrates the basic AWT objects.
 *  with special support for printing
 *  @version 1.1
 *  @author M. Daconta.
 */
import java.awt.*;
import java.awt.event.*;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Properties;

/**
 * Class to demonstrate AWT components.
 */
class ComponentHolder extends Frame
implements ActionListener
{
    /** adapter class to handle window close events. */
    class ShutdownAdapter extends WindowAdapter
    {
        public void windowClosing(WindowEvent e)
        { System.exit(0); }
    }

	public void actionPerformed(ActionEvent ae)
    {
        String sCommand = ae.getActionCommand();
        if ("Exit".equals(sCommand))
        {
            dispose();
        }
		else if ("Print".equals(sCommand))
		{
			Properties printProps = new Properties();
			PrintJob pJob = getToolkit().getPrintJob(this,
													"Printing Test", printProps);

			// null means user canceled print job
			if (pJob != null)
			{
				Graphics pg = pJob.getGraphics();
				if (pg != null)
				{
					printAll(pg);
					pg.dispose(); // flush page
				}
				pJob.end();
				System.out.println("Print job properties");
				printProps.list(System.out);

			}

		}
	}


    /** constructor. */
    ComponentHolder()
    {
        super("AWT Basic Components");


        MenuBar mBar = new MenuBar();
        setMenuBar(mBar);
        Menu fileMenu = new Menu("File");
        mBar.add(fileMenu);
        MenuItem mi = new MenuItem("Print");
        mi.addActionListener(this);
        fileMenu.add(mi);
        mi = new MenuItem("Exit");
        mi.addActionListener(this);
        fileMenu.add(mi);


		/*
		 *  Use a border layout for the set of panels
		 */
        setLayout(new BorderLayout());
        setBackground(Color.lightGray);

        addWindowListener(new ShutdownAdapter());

		/*
		 * North (or Top) panel contains
		 * a button,
		 * a label
		 * and a checkbox.
		 */
        Panel northP = new Panel();
        northP.setLayout(new FlowLayout(FlowLayout.LEFT));
        northP.add(new Button("my Button"));
        northP.add(new Label("my Label"));
        northP.add(new Checkbox("my CheckBox"));
        add("North", northP);

		/*
		*  Center panel contains
		* a list box with three items
		* a choice box with three items
		* and a white canvas
		*/
        Panel centerP = new Panel();
        centerP.setLayout(new FlowLayout(FlowLayout.LEFT));
        List l = null;
        centerP.add(l = new List(5,true));
        l.add("listItem1");
        l.add("listItem2");
        l.add("listItem3");

        Choice c = null;
        centerP.add(c = new Choice());
        c.add("choiceItem1");
        c.add("choiceItem2");
        c.add("choiceItem3");
        Canvas can;
        centerP.add(can = new Canvas());
        can.setBackground(Color.white);
		can.setSize(80,80);
		add("Center", centerP);

		/*
		* The south panel contains
		* A text field
		* and a text area with scroll bars.
		*/
        Panel southP = new Panel();
        southP.setLayout(new FlowLayout(FlowLayout.LEFT));
        southP.add(new TextField("my text field."));
        southP.add(new TextArea("my text Area."));
        southP.add(new Scrollbar(Scrollbar.VERTICAL));
        add("South", southP);

        pack();
        show();
    }

    /** main() method to invoke from JVM. */
    public static void main(String args[])
    {
        new ComponentHolder();
    }
}
