
package jwiley.chp10;

import java.awt.*;
import java.awt.dnd.*;

/**
 * This class extends functionality of
 * java.awt.TextArea to act as a DragSource
 *
 * @author Eric Monk
 * @version 1.1
 */

public class DragSourceTextArea extends TextArea
                implements DragGestureListener, DragSourceListener
{
    public DragSourceTextArea ()
    {
        DragSource ds = DragSource.getDefaultDragSource();
        ds.createDefaultDragGestureRecognizer(this, DnDConstants.ACTION_COPY, this);
    }

    public void dragGestureRecognized(DragGestureEvent dge)
    {
        String sCopyText = getText();
        if ((sCopyText != null) && (!sCopyText.equals("")))
        {
            AsciiTextFlavor sDragDropText =
                              new AsciiTextFlavor(sCopyText);

            dge.startDrag(DragSource.DefaultCopyNoDrop,sDragDropText,this);
        }
    }

    public void dragEnter (DragSourceDragEvent dsde)
    {
        DragSourceContext dsc = dsde.getDragSourceContext();
        dsc.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
    }

    public void dragOver (DragSourceDragEvent dsde)
    {
        DragSourceContext dsc = dsde.getDragSourceContext();
        dsc.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
    }

    public void dropActionChanged (DragSourceDragEvent dsde)  { }

    public void dragExit (DragSourceEvent dse)
    {
        DragSourceContext dsc = dse.getDragSourceContext();
        dsc.setCursor(DragSource.DefaultCopyNoDrop);
    }

    public void dragDropEnd (DragSourceDropEvent dsde) { }

    public static void main (String args[])
    {
        Frame f = new Frame("Simple Drag and Drop");
        f.setLayout(new GridLayout(0,1));

        DragSourceTextArea dsTextArea = new DragSourceTextArea();
        DropTargetTextArea dtTextArea = new DropTargetTextArea();

        f.add(dsTextArea);
        f.add(dtTextArea);

        f.setSize(300,300);
        f.show();
    }
}