
package jwiley.chp10;

import java.awt.*;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.dnd.*;
import java.io.InputStream;

/**
 * This class extends functionality of
 * java.awt.TextArea to act as a DropTarget
 *
 * @author Eric Monk
 * @version 1.1
 */

public class DropTargetTextArea extends TextArea
                                implements DropTargetListener
{
    public DropTargetTextArea ()
    {
        DropTarget dt = new DropTarget(this, this);
        setDropTarget(dt);
    }

    public void dragEnter (DropTargetDragEvent dtde)
    {
        DataFlavor[] flavors = dtde.getCurrentDataFlavors();
        if (flavors != null)
        {
            for (int iCount = 0; iCount < flavors.length; iCount++)
            {
                if (flavors[iCount].equals(DataFlavor.plainTextFlavor))
                {
                    int iSourceActions = dtde.getSourceActions();
                    if ((iSourceActions & DnDConstants.ACTION_COPY)
                                                == DnDConstants.ACTION_COPY)
                    {
                        dtde.acceptDrag(DnDConstants.ACTION_COPY);
                        return;
                    }
                }
                else if (flavors[iCount].equals(DataFlavor.stringFlavor))
                {
                    int iSourceActions = dtde.getSourceActions();
                    if ((iSourceActions & DnDConstants.ACTION_COPY)
                                                == DnDConstants.ACTION_COPY)
                    {
                        dtde.acceptDrag(DnDConstants.ACTION_COPY);
                        return;
                    }
                }
            }
        }
        dtde.rejectDrag();
    }

    public void dragOver (DropTargetDragEvent dtde) { }
    public void dragExit (DropTargetEvent dte)  { }
    public void dropActionChanged(DropTargetDragEvent dtde) { }

    public void drop (DropTargetDropEvent dtde)
    {
        Transferable trans = dtde.getTransferable();
        DataFlavor flavors[] = dtde.getCurrentDataFlavors();
        Object oDropObj = null;

        try
        {
            for (int iCount = 0; iCount < flavors.length; iCount++)
            {
                if (flavors[iCount].equals (DataFlavor.plainTextFlavor) ||
                    flavors[iCount].equals (DataFlavor.stringFlavor))
                {
                    dtde.acceptDrop (DnDConstants.ACTION_COPY);
                    oDropObj = trans.getTransferData(flavors[iCount]);
                    break;
                }
            }

            if (oDropObj != null && oDropObj instanceof InputStream)
            {
                InputStream inStream = (InputStream) oDropObj;
                StringBuffer sb = new StringBuffer();
                byte[] b = new byte[64];
                int iCount = inStream.read(b);
                while (iCount != -1)
                {
                    sb.append (new String (b, 0, iCount));
                    iCount = inStream.read(b);
                }
                inStream.close();
                this.setText (sb.toString());
                dtde.getDropTargetContext().dropComplete(true);
            }
            else if (oDropObj != null && oDropObj instanceof String)
            {
                this.setText ((String)oDropObj);
                dtde.getDropTargetContext().dropComplete(true);
            }
            else
            {
                dtde.rejectDrop ();
                dtde.getDropTargetContext().dropComplete(false);
            }

        } catch (Exception e)
          {
            e.printStackTrace();
          }
    }

    public static void main (String args[])
    {
        Frame f = new Frame("Drop Target");
        f.setLayout(new GridLayout(0,1));

        DropTargetTextArea dsTextArea = new DropTargetTextArea();
        f.add(dsTextArea);

        f.setSize(300,300);
        f.show();
    }

}
