package jwiley.chp10;

import java.awt.*;
import java.awt.event.*;
import java.util.EventObject;
import java.util.Vector;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.io.IOException;

/**
 *  EventTester.java
 *  A class that demonstrates event firing and capturing.
 *  @version 1.1
 *  @author M. Daconta.
 */


class EventTester extends Frame implements
	ActionListener,
	FocusListener,
    KeyListener,
	MouseListener,
    MouseMotionListener,
	WindowListener,
    ItemListener
{
    /** flag to denote events should be saved. */
    private boolean capture;
    /** flag to denote events should be printed. */
    private boolean print;
    /** Vector to store events. */
    private Vector evtVector;

    /** constructor. */
    EventTester()
    {
        super("Event Tester");
        capture = print = false;
        evtVector = new Vector(100);

        MenuBar mBar = new MenuBar();
        setMenuBar(mBar);
        Menu fileMenu = new Menu("File");
        mBar.add(fileMenu);
        CheckboxMenuItem cbmi = new CheckboxMenuItem("Capture Events");
        cbmi.addItemListener(this);
        CheckboxMenuItem cbmi2 = new CheckboxMenuItem("Print Events");
        cbmi2.addItemListener(this);
        MenuItem mi = new MenuItem("Dump Events To File");
        mi.addActionListener(this);
        MenuItem mi2 = new MenuItem("Exit");
        mi2.addActionListener(this);
        fileMenu.add(cbmi);
        fileMenu.add(cbmi2);
        fileMenu.add(mi);
        fileMenu.add(mi2);

        setLocation(new Point(100,100));
        setSize(300,100);



		/*
		 *  Use a border layout for a set of panels
		 */
        setLayout(new BorderLayout());

		/*
		 * North (or Top) panel contains
		 * a button,
		 * a label
		 * and a checkbox.
		 */
        Panel northP = new Panel();
        northP.setLayout(new FlowLayout(FlowLayout.LEFT));
		Button b = new Button("my Button");
		b.addActionListener(this);
        northP.add(b);
		Checkbox cb = new Checkbox("my CheckBox");
		cb.addItemListener(this);
        northP.add(cb);

        add("North", northP);



        // add other event listeners
        addFocusListener(this);
        addKeyListener(this);
        addMouseListener(this);
        addMouseMotionListener(this);
        addWindowListener(this);

        show();
    }

    /** utility routine. */
    public void checkCapture(EventObject eo)
    {
        if (capture)
        {
            evtVector.addElement(eo);
        }
    }

    /** ActionListener method. */
    public void actionPerformed(ActionEvent ae)
    {
        String sCommand = ae.getActionCommand();
        System.out.println("Action Performed: " + sCommand);
        if ("Exit".equals(sCommand))
        {
            dispose();
        }
        else if ("Dump Events To File".equals(sCommand))
        {
            if (evtVector.size() > 0)
            {
                try
                {
                    FileWriter fw = new FileWriter("Event.txt");
                    PrintWriter pw = new PrintWriter(fw);
					pw.println("Dumping " + evtVector.size() + " events:");
                    for (int i=0; i < evtVector.size(); i++)
					{
                    	pw.println(((EventObject)evtVector.elementAt(i)).toString());
					}
                    fw.close();
                }
				catch (IOException ioe)
                {
                    System.out.println(ioe.toString());
                    System.out.println("Unable to dump to file.");
                }
            }
            else
            {
                System.out.println("event vector is empty.");
            }
        }

        checkCapture(ae);
    }

    /** ItemListener method. */
    public void itemStateChanged(ItemEvent ie)
    {
        String sItem = (String) ie.getItem();
        System.out.println("Item state changed: " + sItem);
        if ("Capture Events".equals(sItem) &&
             ie.getStateChange() == ie.SELECTED)
        {
            capture = !capture;
        }
        else if ("Print Events".equals(sItem) &&
                 ie.getStateChange() == ie.SELECTED)
        {
            print = !print;
        }
        checkCapture(ie);
    }

    /** FocusListener method. */
    public void focusGained(FocusEvent fe)
    {
        if (print)
            System.out.println("Gained Focus.");
        checkCapture(fe);
    }

    /** FocusListener method. */
    public void focusLost(FocusEvent fe)
    {
        if (print)
            System.out.println("Lost Focus.");
        checkCapture(fe);
    }

    /** KeyListener method. */
    public void keyPressed(KeyEvent ke)
    {
        if (print)
            System.out.println("Key Pressed.");
        checkCapture(ke);
    }

    /** KeyListener method. */
    public void keyReleased(KeyEvent ke)
    {
        if (print)
            System.out.println("Key Released.");
        checkCapture(ke);
    }

    /** KeyListener method. */
    public void keyTyped(KeyEvent ke)
    {
        if (print)
        {
            System.out.println("Key Typed.");
            System.out.println("Key char: " + ke.getKeyChar());
        }
        checkCapture(ke);
    }

    /** MouseListener method. */
    public void mouseClicked(MouseEvent me)
    {
        if (print)
        {
            System.out.println("Mouse Clicked.");
            System.out.println("At (x,y): " + "(" +
                               me.getX() + "," + me.getY() +
                               ")");
        }
        checkCapture(me);
    }

    /** MouseListener method. */
    public void mouseEntered(MouseEvent me)
    {
        if (print)
            System.out.println("Mouse Entered.");
        checkCapture(me);
    }

    /** MouseListener method. */
    public void mouseExited(MouseEvent me)
    {
        if (print)
            System.out.println("Mouse Exited.");
        checkCapture(me);
    }

    /** MouseListener method. */
    public void mousePressed(MouseEvent me)
    {
        if (print)
            System.out.println("Mouse Pressed.");
        checkCapture(me);
    }

    /** MouseListener method. */
    public void mouseReleased(MouseEvent me)
    {
        if (print)
            System.out.println("Mouse Released.");
        checkCapture(me);
    }

    /** MouseMotionListener method. */
    public void mouseDragged(MouseEvent me)
    {
        if (print)
            System.out.println("Mouse Dragged.");
        checkCapture(me);
    }

    /** MouseMotionListener method. */
    public void mouseMoved(MouseEvent me)
    {
        if (print)
            System.out.print(".");
        checkCapture(me);
    }

    /** WindowListener method. */
    public void windowActivated(WindowEvent we)
    {
        if (print)
            System.out.println("Window Activated.");
        checkCapture(we);
    }

    /** WindowListener method. */
    public void windowClosed(WindowEvent we)
    {
        if (print)
            System.out.println("Window Closed.");
        checkCapture(we);
        System.exit(0);
    }

    /** WindowListener method. */
    public void windowClosing(WindowEvent we)
    {
        if (print)
            System.out.println("Window Closing.");
        checkCapture(we);
        dispose();
    }

    /** WindowListener method. */
    public void windowDeactivated(WindowEvent we)
    {
        if (print)
            System.out.println("Window Deactivated.");
        checkCapture(we);
    }

    /** WindowListener method. */
    public void windowDeiconified(WindowEvent we)
    {
        if (print)
            System.out.println("Window Deiconified.");
        checkCapture(we);
    }

    /** WindowListener method. */
    public void windowIconified(WindowEvent we)
    {
        if (print)
            System.out.println("Window Iconified.");
        checkCapture(we);
    }

    /** WindowListener method. */
    public void windowOpened(WindowEvent we)
    {
        if (print)
            System.out.println("Window Opened.");
        checkCapture(we);
    }

    /** main() method to invoke from the JVM. */
    public static void main(String args[])
    {
        new EventTester();
    }
}
