
package jwiley.chp10;

import java.awt.*;
import java.util.Vector;
import java.util.StringTokenizer;

public class GraphPaperLayout implements LayoutManager
{
	Vector vLayoutElements = new Vector();
    int iRows;
    int iCols;
    int iXInsets;
    int iYInsets;

    public GraphPaperLayout (int iCols, int iRows)
    {
        this(iCols, iRows, 0, 0);
    }

    public GraphPaperLayout (int iCols, int iRows, int iXInsets, int iYInsets)
    {
        if (iRows < 1)
            iRows = 1;
        else
            this.iRows = iRows;

        if (iCols < 1)
            iCols = 1;
        else
            this.iCols = iCols;

        if (iXInsets < 0)
            iXInsets = 0;
        else
            this.iXInsets = iXInsets;

        if (iYInsets < 0)
            iYInsets = 0;
        else
            this.iYInsets = iYInsets;
    }

    public void addLayoutComponent(String sCoordinates, Component comp)
    {
		GraphPaperLayoutElement layoutElement =
		    new GraphPaperLayoutElement(sCoordinates, comp);
		vLayoutElements.addElement(layoutElement);
    }

    public void removeLayoutComponent(Component comp)
    {
		for (int iCount = 0; iCount < vLayoutElements.size(); iCount++)
		{
			Component compToTest =
			  ((GraphPaperLayoutElement) vLayoutElements.elementAt(iCount)).comp;
			if (compToTest == comp)
			{
				vLayoutElements.removeElementAt(iCount);
				break;
			}
		}
    }

    public Dimension preferredLayoutSize(Container target)
    {
        return (getLayoutSize(target, false));
    }

    public Dimension minimumLayoutSize(Container target)
    {
        return (getLayoutSize(target, true));
    }

    private Dimension getLayoutSize (Container target, boolean bGetMinimum)
    {
        int     iMaxHeight = 0;
        int     iMaxWidth = 0;

        // find out largest preferred size for width and height
        int iNumComponents = vLayoutElements.size();
        for (int iCount = 0; iCount < iNumComponents; iCount++)
        {
			GraphPaperLayoutElement layoutElement =
			    (GraphPaperLayoutElement) vLayoutElements.elementAt(iCount);
            Component comp = target.getComponent(iCount);
            Dimension d = null;
            if (bGetMinimum)
                d = comp.getMinimumSize();
            else
                d = comp.getPreferredSize();

			int iWidthToTest = (int)((((iCols * d.width) /
			                    layoutElement.iWidth) / iCols) + 1);
			int iHeightToTest = (int)((((iRows * d.height) /
			                    layoutElement.iHeight) / iRows) + 1);

            if (iWidthToTest > iMaxWidth)
                iMaxWidth = iWidthToTest;

            if (iHeightToTest > iMaxHeight)
                iMaxHeight = iHeightToTest;
        }

        // add grid insets
        iMaxHeight += iYInsets * 2;
        iMaxWidth += iXInsets * 2;

        // multiply by cols and rows
        iMaxHeight *= iRows;
        iMaxWidth *= iCols;

        // add target insets
        Insets insets = target.getInsets();

        iMaxHeight += insets.bottom + insets.top;
        iMaxWidth += insets.left + insets.right;

        return (new Dimension(iMaxWidth, iMaxHeight));
    }

    public void layoutContainer(Container target)
    {
        Insets insets = target.getInsets();
        int iNumComponents = vLayoutElements.size();
        Dimension d = target.getSize();
        d.width -= insets.left + insets.right;
        d.height -= insets.top + insets.bottom;

        float fCellWidth = (float) (d.width * 1.0 / iCols);
        float fCellHeight = (float) (d.height * 1.0 / iRows);

        for (int iCount = 0; iCount < iNumComponents; iCount++)
        {
            Component comp = target.getComponent(iCount);
            Dimension pf = comp.getPreferredSize();
			GraphPaperLayoutElement layoutElement =
			    (GraphPaperLayoutElement) vLayoutElements.elementAt(iCount);

            int iLeft = layoutElement.iLeft;
            int iTop = layoutElement.iTop;
            int iWidth = layoutElement.iWidth;
			int iHeight = layoutElement.iHeight;
            boolean bUsePreferredSize = layoutElement.bUsePreferredSize;

            int x1 = 0, y1 = 0, x2 = 0, y2 = 0;

            if (bUsePreferredSize)
            {
                int iWholeWidth = (int)(fCellWidth * iWidth) - (2 * iXInsets);
                int iWholeHeight = (int)(fCellHeight * iHeight) - (2 * iYInsets);

				x1 = (int)(fCellWidth * (iLeft - 1)) +
				    insets.left + iXInsets + ((iWholeWidth - pf.width) / 2);
                y1 = (int)(fCellHeight * (iTop - 1)) +
                    insets.top + iYInsets + ((iWholeHeight - pf.height) / 2);
                x2 = pf.width;
                y2 = pf.height;
            }
            else
            {
                x1 = (int)(fCellWidth * (iLeft - 1)) + insets.left + iXInsets;
                y1 = (int)(fCellHeight * (iTop - 1)) + insets.top + iYInsets;
                x2 = (int)(fCellWidth * iWidth) - (2 * iXInsets);
                y2 = (int)(fCellHeight * iHeight) - (2 * iYInsets);
            }

            comp.setBounds(x1, y1, x2, y2);

        }
    }

    public String toString()
    {
        return getClass().getName();
    }
}

class GraphPaperLayoutElement
{
	Component	comp;
	int			iLeft;
	int			iTop;
	int			iWidth;
	int			iHeight;
	boolean		bUsePreferredSize = false;

	GraphPaperLayoutElement(String sCoordinates, Component comp)
	{
		this.comp = comp;
        StringTokenizer st = new StringTokenizer(sCoordinates, ",;\t");

        iLeft = Integer.valueOf(st.nextToken()).intValue();
        iTop = Integer.valueOf(st.nextToken()).intValue();

        if (st.hasMoreElements())
        {
            String sNextElement = st.nextToken();
            if (sNextElement.equals("p"))
            {
                bUsePreferredSize = true;
                iWidth = 1;
                iHeight = 1;
            }
            else
            {
                iWidth = Integer.valueOf(sNextElement).intValue();
                iHeight = Integer.valueOf(st.nextToken()).intValue();
                if (st.hasMoreElements())
                {
                    sNextElement = st.nextToken();
                    if (sNextElement.equals("p"))
                        bUsePreferredSize = true;
                }
            }
        }
        else
        {
            iWidth = 1;
            iHeight = 1;
        }
	}
}

