
package jwiley.chp10;

import java.awt.*;
import java.awt.geom.*;
import java.awt.event.*;
import java.util.StringTokenizer;
import java.util.Vector;

import jwiley.chp10.GraphPaperLayout;

public class Graphics2DShapeTester extends Frame implements ActionListener
{
    Choice      choShape = new Choice();
    Button      btnDraw = new Button("Draw");
    Button      btnClear = new Button("Clear");
    Button      btnExit = new Button("Exit");
    TextField   txtParams = new TextField();
    Graphics2DPanel drawArea = new Graphics2DPanel();

    public Graphics2DShapeTester ()
    {
        super ("Graphics2DShapeTester");

        setBackground(Color.lightGray);
        setLayout(new GraphPaperLayout(3,10));

        choShape.add("Rectangle2D");
        choShape.add("RoundRectangle2D");
        choShape.add("Ellipse2D");
        choShape.add("QuadCurve2D");
        choShape.add("CubicCurve2D");

        btnDraw.addActionListener(this);
        btnClear.addActionListener(this);
        btnExit.addActionListener(this);

        // layout form
        add("1,1,3,7", drawArea);
        add("1,8", new Label("Shape"));
        add("2,8,2,1", choShape);
        add("1,9", new Label("Params"));
        add("2,9,2,1", txtParams);
        add("1,10,p", btnDraw);
        add("2,10,p", btnClear);
        add("3,10,p", btnExit);

        setSize(400,400);
        show();
    }

    public void actionPerformed (ActionEvent evt)
    {
        Object oTarget = evt.getSource();
        if (oTarget == btnDraw)
        {
            // check params
            String sParams = txtParams.getText();
            if (sParams.equals(""))
            {
                System.out.println("Must enter some params");
                return;
            }

            StringTokenizer st = new StringTokenizer(sParams, " ,");
            float fParams[] = new float[st.countTokens()];
            int iIndex = 0;
            while (st.hasMoreTokens())
            {
                try
                {
                    fParams[iIndex] = (new Float(st.nextToken())).floatValue();
                    iIndex++;
                }
                catch (NumberFormatException nfe)
                {
                    System.out.println("Params must be floating point values");
                    return;
                }
            }

            // add shape
            ShapeParameters shapeParams =
                new ShapeParameters(choShape.getSelectedItem(), fParams);
            drawArea.addShape(shapeParams);
        }
        else if (oTarget == btnClear)
        {
            drawArea.clear();
        }
        else if (oTarget == btnExit)
        {
            dispose();
            System.exit(1);
        }
    }

    public static void main (String args[])
    {
        new Graphics2DShapeTester();
    }
}

class ShapeParameters
{
    String  sShapeName;
    float   fParams[];

    ShapeParameters (String sShapeName, float fParams[])
    {
        this.sShapeName = sShapeName;
        this.fParams = fParams;
    }

    String getShapeName ()  {   return (sShapeName);    }
    float[] getParams ()    {   return (fParams);       }
}

class Graphics2DPanel extends Panel
{
    Vector vShapes = new Vector();

    public void clear ()
    {
        vShapes.removeAllElements();
        repaint();
    }

    public void addShape (ShapeParameters shape)
    {
        vShapes.addElement(shape);
        repaint();
    }

    public void paint (Graphics g)
    {
        // clear drawing area
        g.setColor(Color.white);
        g.fillRect(-1, -1, getWidth() + 1, getHeight() + 1);
        g.setColor(Color.black);

        // get Graphics 2D object
        Graphics2D g2D = (Graphics2D) g;

        g2D.setStroke(new BasicStroke(2f));

        // draw all shapes
        int iNumShapes = vShapes.size();
        for (int i = 0; i < iNumShapes; i++)
        {
            ShapeParameters params = (ShapeParameters) vShapes.elementAt(i);

            String sShapeName = params.getShapeName();
            float fArgs[] = params.getParams();

            Shape s = null;
            if (sShapeName.equals("Rectangle2D"))
            {
                if (fArgs.length == 4)
                    s = new Rectangle2D.Float(fArgs[0], fArgs[1],
                                              fArgs[2], fArgs[3]);
            }
            else if (sShapeName.equals("RoundRectangle2D"))
            {
                if (fArgs.length == 6)
                    s = new RoundRectangle2D.Float(fArgs[0], fArgs[1],
                                                   fArgs[2], fArgs[3],
                                                   fArgs[4], fArgs[5]);
            }
            else if (sShapeName.equals("Ellipse2D"))
            {
                if (fArgs.length == 4)
                    s = new Ellipse2D.Float(fArgs[0], fArgs[1],
                                            fArgs[2], fArgs[3]);
            }
            else if (sShapeName.equals("QuadCurve2D"))
            {
                if (fArgs.length == 6)
                    s = new QuadCurve2D.Float(fArgs[0], fArgs[1],
                                              fArgs[2], fArgs[3],
                                              fArgs[4], fArgs[5]);
            }
            else if (sShapeName.equals("CubicCurve2D"))
            {
                if (fArgs.length == 8)
                    s = new CubicCurve2D.Float(fArgs[0], fArgs[1],
                                               fArgs[2], fArgs[3],
                                               fArgs[4], fArgs[5],
                                               fArgs[6], fArgs[7]);
            }

            if (s != null)
                g2D.draw(s);
            else
            {
                System.out.println("Invalid # of params: " + fArgs.length);
                vShapes.removeElementAt(i);
                iNumShapes--;
            }
        }
    }

    public void update (Graphics g)
    {
        paint(g);
    }
}