
package jwiley.chp10;

import java.awt.*;

public class RectangleObject extends DrawingObject
{
    Point pAnchor;          // used for initial drawing and resizing
    Point pMoveAnchor;      // used for moving
    int x = 0;
    int y = 0;
    int width = 0;
    int height = 0;

    boolean bUseDirtyRectangle = true;

    SelectionHandle selectionHandles[];

    public RectangleObject(Point pAnchor)
    {
        this.pAnchor = pAnchor;
        x = pAnchor.x;
        y = pAnchor.y;
        color = Color.black;
    }

    public void draw (DrawCanvas drawCanvas, Graphics g)
    {
        if (g != null)
        {
            Color colorCurrent = g.getColor();
            g.setColor(this.getColor());
            g.drawRect(x, y, width, height);
            g.setColor(colorCurrent);
        }
    }

    public void resize (DrawCanvas drawCanvas, Point p)
    {
        Graphics g = drawCanvas.getOffscreenGraphics();
        if (g != null)
        {
            Color c = null;
            if (!bUseDirtyRectangle)
                g.setXORMode(Color.white);
            else
            {
                c = this.getColor();
                this.setColor(Color.white);
            }
            draw(drawCanvas, g);
            computeNewRect(p);
            if (!bUseDirtyRectangle)
                g.setPaintMode();
            else
                this.setColor(c);
            draw(drawCanvas, g);
        }
        drawCanvas.repaint();
    }

    public void resize (DrawCanvas drawCanvas, Point p, Cursor cursor)
    {
        Graphics g = drawCanvas.getOffscreenGraphics();
        Point pAnchor1;
        Point pAnchor2;

        if (g != null)
        {
            deselect(drawCanvas);

            Color c = null;
            if (!bUseDirtyRectangle)
                g.setXORMode(Color.white);
            else
            {
                c = this.getColor();
                this.setColor(Color.white);
            }

            draw(drawCanvas, g);

            int iCursorType = cursor.getType();
            if (iCursorType == Cursor.NW_RESIZE_CURSOR)
            {
                pAnchor = new Point(x + width, y + height);
                computeNewRect(p);
            }
            else if (iCursorType == Cursor.NE_RESIZE_CURSOR)
            {
                pAnchor = new Point(x, y + height);
                computeNewRect(p);
            }
            else if (iCursorType == Cursor.SE_RESIZE_CURSOR)
            {
                pAnchor = new Point(x, y);
                computeNewRect(p);
            }
            else if (iCursorType == Cursor.SW_RESIZE_CURSOR)
            {
                pAnchor = new Point(x + width, y);
                computeNewRect(p);
            }
            else if (iCursorType == Cursor.N_RESIZE_CURSOR)
            {
                computeNewRect(new Point(x, y + height),
                               new Point(x + width, y + height), p);
            }
            else if (iCursorType == Cursor.S_RESIZE_CURSOR)
            {
                computeNewRect(new Point(x, y),
                               new Point(x + width, y), p);
            }
            else if (iCursorType == Cursor.W_RESIZE_CURSOR)
            {
                computeNewRect(new Point(x + width, y),
                               new Point(x + width, y + height), p);
            }
            else if (iCursorType == Cursor.E_RESIZE_CURSOR)
            {
                computeNewRect(new Point(x, y),
                               new Point(x, y + height), p);
            }

            if (!bUseDirtyRectangle)
                g.setPaintMode();
            else
                this.setColor(c);
            draw(drawCanvas, g);
            select(drawCanvas);
        }
        drawCanvas.repaint();
    }

    public void setMoveAnchor (Point p)
    {
        this.pMoveAnchor = p;
    }

    public void move (DrawCanvas drawCanvas, Point p)
    {
        Graphics g = drawCanvas.getOffscreenGraphics();
        if (g != null)
        {
            deselect(drawCanvas);
            Color c = null;
            if (!bUseDirtyRectangle)
                g.setXORMode(Color.white);
            else
            {
                c = this.getColor();
                this.setColor(Color.white);
            }
            draw(drawCanvas, g);

            x = x + (p.x - pMoveAnchor.x);
            y = y + (p.y - pMoveAnchor.y);
            pMoveAnchor = p;

            if (!bUseDirtyRectangle)
                g.setPaintMode();
            else
                this.setColor(c);
            draw(drawCanvas, g);
            select(drawCanvas);
        }
        drawCanvas.repaint();
    }

    public void erase (DrawCanvas drawCanvas)
    {
        Graphics g = drawCanvas.getOffscreenGraphics();
        if (g != null)
        {
            if (!bUseDirtyRectangle)
                g.setXORMode(Color.white);
            else
                this.setColor(Color.white);

            draw(drawCanvas, g);
            g.setPaintMode();
        }
        drawCanvas.repaint();
    }

    public boolean isBorderClicked (DrawCanvas drawCanvas, Point p)
    {
        // check within a 1 pixel buffer if the border was clicked
        int iBuffer = 1;
        Rectangle r[] = new Rectangle[4];
        r[0] = new Rectangle(x - iBuffer, y - iBuffer,
                             width + iBuffer * 2,
                             iBuffer * 2 + 1);
        r[1] = new Rectangle(x + width - iBuffer, y - iBuffer,
                             iBuffer * 2 + 1,
                             height + iBuffer * 2);
        r[2] = new Rectangle(x - iBuffer, y + height - iBuffer,
                             width + iBuffer * 2,
                             iBuffer * 2 + 1);
        r[3] = new Rectangle(x - iBuffer, y - iBuffer,
                             iBuffer * 2 + 1,
                             height + iBuffer * 2);

        for (int iCount = 0; iCount < r.length; iCount++)
        {
            if (isPointInRectangle(r[iCount], p))
                return (true);
        }

        return (false);
    }

    public void select (DrawCanvas drawCanvas)
    {
        // draw selection handles
        int iNumHandles = 4;
        int iHandleSize = 5;
        int iHandleOffset = (int)((iHandleSize - 1) / 2) + 1;
        int iMiddleHandleMultiplier = 6;

        boolean bXMiddleHandles = false;
        boolean bYMiddleHandles = false;

        if (width > (iHandleSize * iMiddleHandleMultiplier))
        {
            bXMiddleHandles = true;
            iNumHandles += 2;
        }

        if (height > (iHandleSize * iMiddleHandleMultiplier))
        {
            bYMiddleHandles = true;
            iNumHandles += 2;
        }

        // put handles at four corners
        selectionHandles = new SelectionHandle[iNumHandles];

        selectionHandles[0] =
                new SelectionHandle(x - iHandleOffset, y - iHandleOffset,
                                    iHandleSize, iHandleSize,
                                    new Cursor(Cursor.NW_RESIZE_CURSOR));
        selectionHandles[1] =
                new SelectionHandle(x + width - iHandleOffset,
                                    y - iHandleOffset,
                                    iHandleSize, iHandleSize,
                                    new Cursor(Cursor.NE_RESIZE_CURSOR));
        selectionHandles[2] =
                new SelectionHandle(x + width - iHandleOffset,
                                    y + height - iHandleOffset,
                                    iHandleSize, iHandleSize,
                                    new Cursor(Cursor.SE_RESIZE_CURSOR));
        selectionHandles[3] =
                new SelectionHandle(x - iHandleOffset,
                                    y + height - iHandleOffset,
                                    iHandleSize, iHandleSize,
                                    new Cursor(Cursor.SW_RESIZE_CURSOR));

        if (bXMiddleHandles)
        {
            int iMidPoint = x + ((int) width / 2);
            selectionHandles[4] =
                new SelectionHandle(iMidPoint - iHandleOffset,
                                    y - iHandleOffset,
                                    iHandleSize, iHandleSize,
                                    new Cursor(Cursor.N_RESIZE_CURSOR));

            selectionHandles[5] =
                new SelectionHandle(iMidPoint - iHandleOffset,
                                    y + height - iHandleOffset,
                                    iHandleSize, iHandleSize,
                                    new Cursor(Cursor.S_RESIZE_CURSOR));

        }

        if (bYMiddleHandles)
        {
            int iMidPoint = y + ((int) height / 2);
            int iOffset = 3;
            if (bXMiddleHandles)
                iOffset = 5;

            selectionHandles[iOffset + 1] =
                new SelectionHandle(x - iHandleOffset,
                                    iMidPoint - iHandleOffset,
                                    iHandleSize, iHandleSize,
                                    new Cursor(Cursor.W_RESIZE_CURSOR));

            selectionHandles[iOffset + 2] =
                new SelectionHandle(x + width - iHandleOffset,
                                    iMidPoint - iHandleOffset,
                                    iHandleSize, iHandleSize,
                                    new Cursor(Cursor.E_RESIZE_CURSOR));

        }

        Graphics g = drawCanvas.getOffscreenGraphics();
        for (int iCount = 0; iCount < selectionHandles.length; iCount++)
        {
            g.fillRect(selectionHandles[iCount].rect.x,
                       selectionHandles[iCount].rect.y,
                       selectionHandles[iCount].rect.width,
                       selectionHandles[iCount].rect.height);
        }
        drawCanvas.repaint();
    }

    public void deselect (DrawCanvas drawCanvas)
    {
        Graphics g = drawCanvas.getOffscreenGraphics();
        g.setXORMode(Color.white);
        for (int iCount = 0; iCount < selectionHandles.length; iCount++)
        {
            g.fillRect(selectionHandles[iCount].rect.x,
                       selectionHandles[iCount].rect.y,
                       selectionHandles[iCount].rect.width,
                       selectionHandles[iCount].rect.height);
        }

        g.setPaintMode();
        draw(drawCanvas, g);
        drawCanvas.repaint();
    }

    public void setHandleCursor(DrawCanvas drawCanvas, Point p)
    {
        if (selectionHandles != null)
        {
            for (int iCount = 0; iCount < selectionHandles.length; iCount++)
            {
                if (isPointInRectangle(selectionHandles[iCount].rect, p))
                {
                    drawCanvas.setCursor(selectionHandles[iCount].cursor);
                    return;
                }
            }
        }

        drawCanvas.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
    }

    boolean isPointInRectangle (Rectangle r, Point p)
    {
        if ((p.x >= r.x) &&
            (p.x <= r.x + r.width) &&
            (p.y >= r.y) &&
            (p.y <= r.y + r.height))
        {
            return (true);
        }
        else
            return (false);
    }

    void computeNewRect(Point p)
    {
        if (p.x < pAnchor.x)
        {
            x = p.x;
            width = pAnchor.x - x;
        }
        else
        {
            x = pAnchor.x;
            width = p.x - x;
        }

        if (p.y < pAnchor.y)
        {
            y = p.y;
            height = pAnchor.y - y;
        }
        else
        {
            y = pAnchor.y;
            height = p.y - y;
        }


    }

    void computeNewRect (Point pAnchor1, Point pAnchor2, Point p)
    {
        // assumption - either the x or y of the anchors is the same
        if (pAnchor1.x == pAnchor2.x)
            p.y = pAnchor1.y;
        else
            p.x = pAnchor1.x;

        this.pAnchor = pAnchor2;
        computeNewRect(p);
    }
}


