package jwiley.chp10;

import java.awt.*;
import java.awt.event.*;
/**
 *  TestWindows.java
 *  Class to demonstrate AWT windows.
 *  @version 1.1
 *  @author M. Daconta.
 */

/**
 * 
 */
class TestWindows extends Frame implements ActionListener
{
    /** adapter class to catch window close event. */
    class ShutdownAdapter extends WindowAdapter
    {
        public void windowClosing(WindowEvent we)
        { System.exit(0); }
    }

    /** constructor. */
    TestWindows()
    {
        super("AWT Windows");

        MenuBar mb = new MenuBar();
        setMenuBar(mb);

        Menu fileMenu = new Menu("File");
        mb.add(fileMenu);
        MenuItem mi = new MenuItem("Frame");
        mi.addActionListener(this);
        fileMenu.add(mi);
        mi = new MenuItem("Window");
        mi.addActionListener(this);
        fileMenu.add(mi);
        mi = new MenuItem("Modeless Dialog");
        mi.addActionListener(this);
        fileMenu.add(mi);
        mi = new MenuItem("Modal Dialog");
        mi.addActionListener(this);
        fileMenu.add(mi);
        mi = new MenuItem("FileDialog");
        mi.addActionListener(this);
        fileMenu.add(mi);

        mi = new MenuItem("PopupMenu");
        mi.addActionListener(this);
        fileMenu.add(mi);

        mi = new MenuItem("Exit");
        mi.addActionListener(this);
        fileMenu.add(mi);

        // add WindowListener
        addWindowListener(new ShutdownAdapter());

        setLocation(new Point(100,100));
        setSize(200,200);
        setVisible(true);
    }

    /** ActionListener method. */
    public void actionPerformed(ActionEvent evt)
    {
        String command = evt.getActionCommand();
        if ("Exit".equals(command))
                System.exit(0);
        else if ("Frame".equals(command))
                new bareFrame();
        else if ("Window".equals(command))
                new bareWindow(this);
        else if ("Modeless Dialog".equals(command))
                new bareDialog(this,false,"Modeless Dialog");
        else if ("Modal Dialog".equals(command))
                new bareDialog(this,true,"Modal Dialog");
        else if ("FileDialog".equals(command))
                new bareFileDialog(this);
		else if ("PopupMenu".equals(command))
		{
			bareFrame bf = new bareFrame();
			barePopupMenu bp =  new barePopupMenu(bf);
			bf.add(bp);
			bp.show(bf,10,10);
		}
    }

    /** main method to invoke from JVM. */
    public static void main(String args[])
    {
        new TestWindows();
    }
}

/** adapter class for window listening. */
class DisposeAdapter extends WindowAdapter
{
    public void windowClosing(WindowEvent we)
    {
        we.getWindow().dispose();
    }
}

/** Class to demonstrate a Frame. */
class bareFrame extends Frame
{
    /** constructor. */
    bareFrame()
    {
        super("A Frame");
        setLocation(new Point(110,150));
        setSize(150,150);
        addWindowListener(new DisposeAdapter());
        setVisible(true);
    }
}


/** Class to demonstrate a Window. */
class bareWindow extends Window
{
    /** constructor. */
    bareWindow(Frame parent)
    {
        super(parent);
        setLocation(new Point(110,150));
        setSize(150,150);
        addWindowListener(new DisposeAdapter());
        setVisible(true);
    }
}

/** Class to demonstrate a dialog. */
class bareDialog extends Dialog
{
    /** constructor. */
    bareDialog(Frame parent, boolean modal, String title)
    {
        super(parent,modal);
        setTitle(title);
        setLocation(new Point(110,150));
        setSize(200,150);
        addWindowListener(new DisposeAdapter());
        setVisible(true);
    }
}

/** Class to demonstrate a FileDialog. */
class bareFileDialog extends FileDialog
{
    /** constructor. */
    bareFileDialog(Frame parent)
    {
        super(parent,"A File Dialog");
        setLocation(new Point(110,150));
        addWindowListener(new DisposeAdapter());
        setVisible(true);
    }
}

/** Class to demonstrate a PopupMenu */
class barePopupMenu extends PopupMenu
implements ActionListener
{
	barePopupMenu(Frame _parent)
	{
		super("Help");

		MenuItem item;
		item = new MenuItem("Contents");
		item.addActionListener(this);
		add(item);

		item = new MenuItem("Search for help on");
		item.addActionListener(this);
		add(item);
		addSeparator();

		item = new MenuItem("Tip O'The day");
		item.addActionListener(this);
		add(item);
		addSeparator();
		item = new MenuItem("About TestWindows.java");
		item.addActionListener(this);
		add(item);
	}
	
	public void actionPerformed(ActionEvent ae)
	{
		String command = ae.getActionCommand();
		if ( command.equals("Contents"))
			System.out.println("User selected contents");
		else if ( command.equals("Search for help on"))
			System.out.println("User selected search for help on");
		else if ( command.equals("Tip O'The day"))
			System.out.println("User selected Tip O'The day");
		else if ( command.equals("About TestWindows.java"))
			System.out.println("User selected About TestWindows.java");
	}
}
