
package jwiley.chp12;

import java.net.ServerSocket;
import java.net.Socket;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.FileInputStream;
import java.io.File;
import java.io.InputStreamReader;
import java.io.BufferedReader;

import java.util.Vector;
import java.util.StringTokenizer;

public class SearchServer extends Thread
{
    int iPort = 0;

    public SearchServer (int iPort)
    {
        this.iPort = iPort;
    }

    public void run ()
    {
        ServerSocket sockServer = null;

        try {
            sockServer = new ServerSocket(iPort);
        } catch (Exception e)
          {
            System.out.println("Error creating ServerSocket." +
                               "\nError: " + e.toString());
            System.exit(0);
          }


        try {
            while (true)
            {
                Socket sockConnection = sockServer.accept();
                DataInputStream disStream =
                    new DataInputStream(sockConnection.getInputStream());

                DataOutputStream dosStream =
                    new DataOutputStream(sockConnection.getOutputStream());

                String sCommand = disStream.readUTF();
                if (sCommand.equals("<SEARCH>"))
                {
                    handleSearch(disStream, dosStream);
                }
                else if (sCommand.equals("<GET-DOC>"))
                {
                    handleGetDoc(disStream, dosStream);
                }

                sockConnection.close();
            }

        } catch (IOException ioe)
          {
            System.out.println(ioe.toString());
          }

    }

    void handleSearch (DataInputStream disStream,
                       DataOutputStream dosStream)
    {
        try {
            String sSearchText = disStream.readUTF();
            boolean bTitleSearch = disStream.readBoolean();
            boolean bKeywordSearch = disStream.readBoolean();
            boolean bTextSearch = disStream.readBoolean();

            Document docs[] = parseDocuments(sSearchText,
                                           bTitleSearch,
                                           bKeywordSearch,
                                           bTextSearch);

            if (docs != null)
            {
                dosStream.writeInt(docs.length);
                for (int iCount = 0; iCount < docs.length; iCount++)
                {
                    dosStream.writeUTF(docs[iCount].getFileName());
                    dosStream.writeUTF(docs[iCount].getTitle());
                }
            }
            else
                dosStream.writeInt(0);

        } catch (IOException ioe)
          {
            System.out.println("Error handling search." +
                               "\nError: " + ioe.toString());
          }

    }

    void handleGetDoc (DataInputStream disStream,
                       DataOutputStream dosStream)
    {
        try {
            String sFileName = disStream.readUTF();

            // get file text here
            FileInputStream fis = new FileInputStream(sFileName);
            BufferedReader input =
                new BufferedReader(new InputStreamReader(fis));

            String sDocumentText = "";
            String sLine = input.readLine();

            while (sLine != null)
            {
                if (!sDocumentText.equals(""))
                    sDocumentText = sDocumentText + "\n" + sLine;
                else
                    sDocumentText = sDocumentText + sLine;

                sLine = input.readLine();
            }

            fis.close();

            dosStream.writeUTF(sDocumentText);

        } catch (IOException ioe)
          {
            System.out.println("Error getting document." +
                               "\nError: " + ioe.toString());
          }

    }

    Document[] parseDocuments(String sSearchText,
                              boolean bTitleSearch,
                              boolean bKeywordSearch,
                              boolean bTextSearch)
    {
        File fCurrentDir = new File(".");
        String sFiles[] = fCurrentDir.list();

        Vector vFileHits = new Vector();

        if (sFiles != null)
        {
            for (int iCount = 0; iCount < sFiles.length; iCount++)
            {
                String sCompareString = sFiles[iCount].toLowerCase();
                if (sCompareString.endsWith(".txt"))
                {
                    String sTitle = parseDocument(sFiles[iCount],
                                                  sSearchText,
                                                  bTitleSearch,
                                                  bKeywordSearch,
                                                  bTextSearch);
                    if (sTitle != null)
                    {
                        Document doc = new Document(sFiles[iCount],
                                                    sTitle);
                        vFileHits.addElement(doc);
                    }
                }
            }
        }

        Document documentHits[] = null;

        int iNumHits = vFileHits.size();
        if (iNumHits > 0)
        {
            documentHits = new Document[iNumHits];
            for (int iCount = 0; iCount < iNumHits; iCount++)
            {
                documentHits[iCount] =
                        (Document) vFileHits.elementAt(iCount);
            }
        }

        return (documentHits);
    }

    String parseDocument(String sFileName,
                         String sSearchText,
                         boolean bTitleSearch,
                         boolean bKeywordSearch,
                         boolean bTextSearch)
    {
        try {
            FileInputStream fis = new FileInputStream(sFileName);
            BufferedReader input =
                    new BufferedReader(new InputStreamReader(fis));

            String sTitleString = input.readLine();
            if (bTitleSearch)
            {
                if (parseLine(sTitleString, sSearchText, "Title:"))
                {
                    fis.close();
                    return (sTitleString);
                }
            }

            String sKeywordString = input.readLine();
            if (bKeywordSearch)
            {
                if (parseLine(sKeywordString, sSearchText, "Keywords:"))
                {
                    fis.close();
                    return (sTitleString);
                }
            }

            if (bTextSearch)
            {
                String sTextString = input.readLine();
                while (sTextString != null)
                {
                    if (parseLine(sTextString, sSearchText, null))
                    {
                        fis.close();
                        return (sTitleString);
                    }
                    sTextString = input.readLine();
                }
            }

        } catch (IOException ioe)
          {
            System.out.println("Error parsing text file." +
                               "\nError: " + ioe.toString());
          }

       return (null);
    }

    boolean parseLine(String sLine,
                      String sSearchText,
                      String sPrefixText)
    {
        if (sPrefixText != null)
        {
            sLine = sLine.substring(sPrefixText.length());
        }

        StringTokenizer st = new StringTokenizer(sLine, " ,.:;");
        while (st.hasMoreTokens())
        {
            String sToken = st.nextToken().trim();
            if (sSearchText.equalsIgnoreCase(sToken))
                return (true);
        }

        return (false);
    }

    public static void main (String args[])
    {
        SearchServer searchServer = new SearchServer(8511);
        searchServer.start();
    }

}

