// SimpleBounce.java

import java.applet.Applet;
import java.awt.*;
import java.util.Random;

/** class to demonstrate simple game animation. */
public class SimpleBounce extends Applet implements Runnable
{
    final int SOUTHWEST = 0;
    final int SOUTHEAST = 1;
    final int NORTHWEST = 2;
    final int NORTHEAST = 3;

    Point oldP, newP;
    int BresenhamError;
    int Rise, Run;
    int Direction;
    Point wallPoint;
    Color ballColor;
    int speed;
    int Increment;
    int ballSize;
    Thread bouncer=null;
    boolean isStopped=true;

    public void init()
    {
        setBackground(Color.white);
        speed = 1;
        Increment = 2;
        ballSize = 12;
        ballColor = Color.red;
    }
    public void start()
    {
        if (bouncer == null)
            bouncer = new Thread(this);

        isStopped = false;
        bouncer.start();

    }

    public void stop()
    {
        isStopped = true;
    }

    // implement Runnable
    public void run()
    {
        while (!isStopped)
            repaint();
    }

    public void paint(Graphics g)
    {
        Random dice = new Random();

        // clears background
        // launch direction between 30-70
        int degrees = dice.nextInt() % 70;
        if (degrees < 0) degrees = -degrees;
        if (degrees < 30) degrees = 30;

        // convert to radians
        double radians = ((double)degrees/180.0) * Math.PI;

        // launch point is always the bottom center
        oldP = new Point(getSize().width/2, getSize().height -
                                ballSize);
        newP = new Point(getSize().width/2, getSize().height -
                                ballSize);

        // figure out wall intersection
        double oppositte = (double)(getSize().width/2) *
                                Math.tan(radians);

        if (degrees < 90)
                wallPoint = new Point(getSize().width, (int) oppositte);
        else
                wallPoint = new Point(0, (int) oppositte);

        Rise = (int) Math.round(oppositte);
        Run = Math.abs(oldP.x - wallPoint.x);

        // determine direction
        if (wallPoint.y < oldP.y)
        {
                if (wallPoint.x > oldP.x)
                        Direction = NORTHEAST;
                else
                        Direction = NORTHWEST;
        }
        else
        {
                if (wallPoint.x > oldP.x)
                        Direction = SOUTHEAST;
                else
                        Direction = SOUTHWEST;
        }

        if (Rise >= Run)
                BresenhamError = Rise;
        else
                BresenhamError = Run;
    }

    /** method - not used to increase speed. */
    boolean BresenhamIncrement(int majorDimension,
                               int minorDimension)
    {
        if (BresenhamError >= majorDimension)
        {
                BresenhamError = minorDimension;
                return true;
        }
        else
                BresenhamError += minorDimension;

        return false;
    }

    public void update(Graphics g)
    {
        // clear old ball
        g.setColor(Color.white);
        g.fillOval(oldP.x, oldP.y, ballSize, ballSize);

        // draw new ball
        g.setColor(ballColor);
        g.fillOval(newP.x, newP.y, ballSize, ballSize);

        // old now is new
        oldP.x = newP.x;
        oldP.y = newP.y;

        // step, use Bresenham
        if (Rise >= Run)
        {
            switch (Direction) {
                case NORTHEAST:
                        newP.y -= Increment;

                        if (BresenhamError >= Rise)
                        {
                                BresenhamError = Run;
                                newP.x += Increment;
                        }
                        else
                                BresenhamError += Run;
                        break;
                case NORTHWEST:
                        newP.y -= Increment;
                        if (BresenhamError >= Rise)
                        {
                                BresenhamError = Run;
                                newP.x -= Increment;
                        }
                        else
                                BresenhamError += Run;
                        break;
                case SOUTHEAST:
                        newP.y += Increment;
                        if (BresenhamError >= Rise)
                        {
                                BresenhamError = Run;
                                newP.x += Increment;
                        }
                        else
                                BresenhamError += Run;
                        break;
                case SOUTHWEST:
                        newP.y += Increment;
                        if (BresenhamError >= Rise)
                        {
                                BresenhamError = Run;
                                newP.x -= Increment;
                        }
                        else
                                BresenhamError += Run;
                        break;
                default:
                        System.out.println("Unknown Direction");
            }
        }
        else  // Run > Rise
        {
            switch (Direction)
            {
            case NORTHEAST:
                    newP.x += Increment;
                    if (BresenhamError >= Run)
                    {
                            BresenhamError = Rise;
                            newP.y -= Increment;
                    }
                    else
                            BresenhamError += Rise;
                    break;
            case NORTHWEST:
                    newP.x -= Increment;
                    if (BresenhamError >= Run)
                    {
                            BresenhamError = Rise;
                            newP.y -= Increment;
                    }
                    else
                            BresenhamError += Rise;
                    break;
            case SOUTHEAST:
                    newP.x += Increment;
                    if (BresenhamError >= Run)
                    {
                            BresenhamError = Rise;
                            newP.y += Increment;
                    }
                    else
                            BresenhamError += Rise;
                    break;
            case SOUTHWEST:
                    newP.x -= Increment;
                    if (BresenhamError >= Run)
                    {
                            BresenhamError = Rise;
                            newP.y += Increment;
                    }
                    else
                            BresenhamError += Rise;
                    break;
            default:
                    System.out.println("Unknown Direction");
            }
        }

        // now check for a "bounce"
        if ( (newP.x + ballSize) >= getSize().width )
        {
            if (Direction == SOUTHEAST)
                    Direction = SOUTHWEST;
            else
                    Direction = NORTHWEST;
        }
        else if ( newP.x <= 0 )
        {
            if (Direction == SOUTHWEST)
                    Direction = SOUTHEAST;
            else
                    Direction = NORTHEAST;
        }
        else if ( (newP.y + ballSize) >= getSize().height)
        {
            if (Direction == SOUTHWEST)
                    Direction = NORTHWEST;
            else
                    Direction = NORTHEAST;
        }
        else if (newP.y <= 0)
        {
            if (Direction == NORTHWEST)
                    Direction = SOUTHWEST;
            else
                    Direction = SOUTHEAST;
        }
    }
}
