
package jwiley.chp16;

import java.sql.DriverManager;
import java.sql.Connection;
import java.sql.Statement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.DatabaseMetaData;

/**
 * Class to demonstrate jdbc transactions
 *
 * @author  Eric Monk
 */

public class AddBook
{
    public static void main (String args[])
    {
        if (args.length != 3)
        {
            System.out.println("Usage: java AddBook <author> <title> <num pages>");
            System.exit(0);
        }

        try
        {
            // Load database driver
            Class.forName ("sun.jdbc.odbc.JdbcOdbcDriver");
        } catch (ClassNotFoundException cnfe)
          {
            System.out.println(cnfe.toString());
            System.exit(0);
          }

        try
        {
            // Establish connection
            String sUrl = "jdbc:odbc:ExampleDb";
            Connection con = DriverManager.getConnection(sUrl);

            DatabaseMetaData dbmd = con.getMetaData();
            boolean bSupportsTransactions = dbmd.supportsTransactions();

            if (bSupportsTransactions)
            {
	            int iTransactionIsolationLevel = con.getTransactionIsolation();
	            if (iTransactionIsolationLevel == Connection.TRANSACTION_NONE)
	            {
	                // try to set a higher TRANSACTION LEVEL
                    boolean bSupportsTransLevel = dbmd.supportsTransactionIsolationLevel(Connection.TRANSACTION_READ_COMMITTED);
                    if (bSupportsTransLevel)
                        con.setTransactionIsolation(Connection.TRANSACTION_READ_COMMITTED);
                }
	        }

            Statement stmt = con.createStatement();

            try {
                if (bSupportsTransactions)
                {
                    // begin transaction
                    con.setAutoCommit(false);
                }

                // Get new AuthorId
                String sQuery = "SELECT MAX(AuthorId) AS Max_AuthorId FROM AUTHORS";
                ResultSet rslt = stmt.executeQuery(sQuery);

                int iNewAuthorId = 0;
                while (rslt.next())
                {
                    iNewAuthorId = rslt.getInt("Max_AuthorId") + 1;
                }

                // Add to Authors
                String sInsertStmt = "INSERT INTO AUTHORS (AuthorId, Author) VALUES (" +
                                     iNewAuthorId + ", '" + args[0] + "')";
                int iStatus = stmt.executeUpdate(sInsertStmt);

                // Get new BookId
                sQuery = "SELECT MAX(BookId) AS Max_BookId FROM Books_II";
                rslt = stmt.executeQuery(sQuery);

                int iNewBookId = 0;
                while (rslt.next())
                {
                    iNewBookId = rslt.getInt("Max_BookId") + 1;
                }

                // Add to Books_II
                sInsertStmt = "INSERT INTO BOOKS_II (BookId, AuthorId, Title, NumPages) " +
                              "VALUES (" + iNewBookId + ", " + iNewAuthorId +
                              ", '" + args[1] + "', " + args[2] + ")";
                iStatus = stmt.executeUpdate(sInsertStmt);

	            if (bSupportsTransactions)
	            {
	                // commit transaction
                    con.commit();
                }

                rslt.close();

            } catch (SQLException sqle1)
              {
                if (bSupportsTransactions)
                {
                    // rollback transaction
                    con.rollback();
                }
                throw new SQLException(sqle1.getMessage());
              }

            // Cleanup
            stmt.close();

        } catch (SQLException sqle2)
          {
            System.out.println(sqle2.toString());
          }

    }
}
