package jwiley.chp17;
import java.applet.Applet;
import java.awt.Graphics;
import java.awt.Frame;
import java.util.*;
import java.text.*;
import java.awt.event.WindowEvent;
import java.awt.event.WindowAdapter;

/**
 * public class TestFormat
 * @author A. Saganich
 * @version 1.1
 *
 *  Sample applet to exercise the use of Locales.
 *
 *  @see java.util.Locale;
 *  @see java.util.Date;
 *  @see java.text.DateFormat;
 */


public class TestFormat extends Applet 
{
	
	Locale current;
	private Locale supported[] =
	{
		Locale.US,
		Locale.UK,
		Locale.FRENCH,
		Locale.GERMAN
	};
	
	/** Determine what the current locale is */
	public void init() 
	{
		current = getLocale();
	}

	/** 
	* Display the currently supported locales 
	* And test if the current locale is supported.
	*/
	public void paint(Graphics g) 
	{
		int y = 10;
		// First dates
		g.drawString("Current dates by locale are: ",10, y+=10);
		for ( int i = 0; i < supported.length; i++)
		{
			DateFormat df = DateFormat.getDateTimeInstance(DateFormat.LONG,
														   DateFormat.LONG,supported[i]);
			Date date = new Date();
			TimeZone tz = TimeZone.getTimeZone("EST");
			df.setTimeZone(tz);
			String sDate = df.format(date);
			g.drawString(supported[i].getDisplayName()+" " + sDate , 10, y+=10);
		}		
		
		// Now Numbers
		y+=10;
		g.drawString("Formatted numbers by locale: ",10, y+=10);
		for ( int i = 0; i < supported.length; i++)
		{
			double aDouble = 12345678.0129;
			NumberFormat currFormatter = NumberFormat.getCurrencyInstance(supported[i]);
			NumberFormat nf = NumberFormat.getInstance(supported[i]);
			String sCurrency = currFormatter.format(aDouble);
			String sDouble = nf.format(aDouble);
			g.drawString(supported[i].getDisplayName()+" Currency:"+sCurrency,20, y+=10);
			g.drawString(supported[i].getDisplayName()+" Integer:"+sDouble,20, y+=10);
		}
		y+=10;
		g.drawString("Formatted Messages: ",10, y+=10);
		
		// Finally messages.
		Object[] args = 
		{ 
			 new Integer(7), 
			 new Double(5), 
			 "Some Text" 
		 }; 

		String sFormatted = MessageFormat.format( 
			 "An integer {0}, followed by a double {1}, followed by a string \"{2}\".",args);
		g.drawString(sFormatted,10, y+=10);
		
	}

	/**
	 * Simple main method to test the applet.
	 */
	static public void main(String args[])
	{
	    class ShutdownAdapter extends WindowAdapter
	    {
	        public void windowClosing(WindowEvent e)
	        { System.exit(0); }
	    }

		Frame	myFrame	= new Frame("TestFormat");
		myFrame.addWindowListener(new ShutdownAdapter());
		TestFormat TestFormat = new TestFormat();
		myFrame.setSize(400,150);
		myFrame.add("Center",TestFormat);
		TestFormat.init();
		TestFormat.start();
		myFrame.show();
		
		
	}
}