// SearchResultsPage.java
package jwiley.chp18;

import jwiley.chp18.html.*;

/**
 * This class represents an html page displaying the
 * results of a search
 *
 * @author  Eric Monk
 * @version 1.0
 */
public class SearchResultsPage extends HtmlPage
{
    public SearchResultsPage (String sServletAddress,
                                BookRecord records[], String sSearchText)
    {
        super();

        HtmlHead head = new HtmlHead("Search results for: " + sSearchText);

        // Body
        HtmlBody body = new HtmlBody();
        HtmlText title = new HtmlText("Books Online", "H2");;

        title.addFormatTag("CENTER");
        body.addComponent(title);

        BooksSearchForm form = new BooksSearchForm(sServletAddress);
        body.addComponent(form);

        body.addHtml("<HR SIZE=2>");

        // format results
        HtmlText subtitle = new HtmlText("Search Results", "H4");
        body.addComponent(subtitle);
        if (records == null || records.length == 0)
        {
            HtmlText nomatches =
                new HtmlText("Sorry, no matches were found.");
            nomatches.addFormatTag("B");
            nomatches.addFormatTag("P");
            body.addComponent(nomatches);
        }
        else
        {
            HtmlTable results = new HtmlTable();
            HtmlTableRow rows[] = new HtmlTableRow[records.length + 1];
            rows[0] = new HtmlTableRow();
            rows[0].addCell(new HtmlTableCell(new HtmlText("Title"), true));
            rows[0].addCell(new HtmlTableCell(new HtmlText("Author"), true));

            for (int iCount = 0; iCount < records.length; iCount++)
            {
                String sQueryString =
                            "?" + BooksExampleHomePage.COMMAND_CONST +
                            "=" + BooksExampleHomePage.GET_COMMAND +
                            "&bookid=" + records[iCount].getBookId();
                HtmlAnchor anchor =
                        new HtmlAnchor("",
                                   new HtmlText(records[iCount].getTitle()),
                                   sServletAddress + sQueryString);

                rows[iCount + 1] = new HtmlTableRow();
                rows[iCount + 1].addCell(new HtmlTableCell(anchor));
                rows[iCount + 1].addCell(new HtmlTableCell(
                                 new HtmlText(records[iCount].getAuthor())));

                results.addRow(rows[iCount + 1]);
            }

            body.addComponent(results);
        }

        setHead(head);
        setBody(body);

    }

}