import java.io.IOException;
import java.util.Enumeration;
import java.util.Vector;
import java.sql.*;

import javax.servlet.*;
import javax.servlet.http.*;

import jwiley.chp18.*;
import jwiley.chp18.html.HtmlPage;

/**
 * This servlet demonstrates some of the power that can be
 * acheived using servlets. This servlet interacts with the
 * database, handles HTML form processing, and constructs
 * dynamic HTML.
 *
 * @author  Eric Monk
 * @version 1.0
 */
public class DatabaseServlet extends HttpServlet
{
    private Connection  con;
    private  Statement   stmt;

    public void init (ServletConfig config) throws ServletException
    {
        super.init(config);

        try
        {
            // Load database driver
            Class.forName ("sun.jdbc.odbc.JdbcOdbcDriver");
        } catch (ClassNotFoundException cnfe)
          {
            String sErrorMsg = "Error loading database drivers. " +
                               "Error: " + cnfe.toString();
            log(sErrorMsg);
            throw new UnavailableException(this, sErrorMsg);
          }

        try
        {
            // Establish connection
            String sUrl = "jdbc:odbc:ExampleDb";
            con = DriverManager.getConnection(sUrl);
            stmt = con.createStatement();
        } catch (SQLException sqle)
          {
            String sErrorMsg = "Could not establish database connection.";
            log(sErrorMsg);
            logSQLException(sqle);
            throw new UnavailableException(this, sErrorMsg);
          }
    }

    public void destroy ()
    {
        try
        {
            // Close database connection
            stmt.close();
            con.close();
        } catch (SQLException sqle)
          {
            log("Could not close database connection.");
            logSQLException(sqle);
          }

        super.destroy();
    }

    private void logSQLException (SQLException sqle)
    {
        while (sqle != null)
        {
            log("SQLState: " + sqle.getSQLState ());
            log("Message:  " + sqle.getMessage ());
            log("ErrorCode:   " + sqle.getErrorCode ());
            sqle = sqle.getNextException ();
        }
    }

    synchronized protected void doGet(HttpServletRequest req,
                                      HttpServletResponse res)
                                      throws ServletException, IOException
    {
        res.setContentType("text/html");

        String sServletAddress = req.getServletPath();
        ServletOutputStream out = res.getOutputStream();

        // parse query string
        String sQueryString = req.getQueryString();
        if (sQueryString == null)
        {
            BooksExampleHomePage homePage =
                    new BooksExampleHomePage(sServletAddress);
            out.print(homePage.getHtml());
        }
        else
        {
            // look for command parameter
            String sCommand =
                req.getParameter(BooksExampleHomePage.COMMAND_CONST);
            if (sCommand == null)
            {
                BooksExampleHomePage homePage =
                        new BooksExampleHomePage(sServletAddress);
                out.print(homePage.getHtml());
            }
            else
            {
                // handle command parameter
                if (sCommand.equals(BooksExampleHomePage.LIST_ALL_COMMAND))
                {
                    String sql = "SELECT * FROM Books";
                    HtmlPage resultsPage = handleSearch(sql, sServletAddress);
                    out.print(resultsPage.getHtml());
                }
                else if (sCommand.equals(BooksExampleHomePage.SEARCH_COMMAND))
                {
                    String sSearchParam = req.getParameter("searchParams");
                    String sSearchText = req.getParameter("SearchText");

                    String sWhereClause = null;
                    if (sSearchParam.equals("author"))
                    {
                        sWhereClause = " WHERE (Author";
                    }
                    else if (sSearchParam.equals("title"))
                    {
                        sWhereClause = " WHERE (Title";
                    }
                    sWhereClause = sWhereClause + " Like '%" +
                                             sSearchText + "%')";

                    String sql = "SELECT * FROM Books" + sWhereClause;

                    HtmlPage resultsPage = handleSearch(sql, sServletAddress);
                    out.print(resultsPage.getHtml());
                }
                else if (sCommand.equals(BooksExampleHomePage.GET_COMMAND))
                {
                    String sBookId = req.getParameter("bookid");
                    String sql = "SELECT * FROM Books WHERE BookId = " + sBookId;

                    try
                    {
                        ResultSet rslt = stmt.executeQuery(sql);

                        // Process data
                        if (rslt.next())
                        {
                            BookRecord record = new BookRecord();
                            record.setBookId(rslt.getInt("BookId"));
                            record.setTitle(rslt.getString("Title"));
                            record.setAuthor(rslt.getString("Author"));
                            record.setNumPages(rslt.getInt("NumPages"));

                            BooksDetailPage detailPage =
                                new BooksDetailPage(sServletAddress, record);
                            out.print(detailPage.getHtml());
                        }
                        else
                        {
                            String sMessage = "Could not find specified record.";
                            MessagePage message =
                                new MessagePage(sServletAddress,
                                                "Error", sMessage);
                            out.print(message.getHtml());
                        }
                        rslt.close();

                    } catch (SQLException sqle)
                      {
                        String sMessage = "A database error has occurred.";
                        MessagePage message =
                            new MessagePage(sServletAddress, "Error",
                                            sMessage + "<BR>Error: " +
                                            sqle.toString());
                        out.print(message.getHtml());
                        logSQLException(sqle);
                      }
                }
                else if (sCommand.equals(BooksExampleHomePage.NEW_COMMAND))
                {
                    // get form in new mode
                    BooksDetailPage newBook =
                            new BooksDetailPage(sServletAddress, null);
                    out.print(newBook.getHtml());
                }
            }
        }
    }

    synchronized protected void doPost(HttpServletRequest req,
                                       HttpServletResponse res)
                                       throws ServletException, IOException
    {
        res.setContentType("text/html");

        String sServletAddress = req.getServletPath();
        ServletOutputStream out = res.getOutputStream();

        // look for command parameter
        String sCommand =
            req.getParameter(BooksExampleHomePage.COMMAND_CONST);
        if (sCommand == null)
        {
            BooksExampleHomePage homePage =
                    new BooksExampleHomePage(sServletAddress);
            out.print(homePage.getHtml());
        }
        else
        {
            // handle command parameter
            if (sCommand.equals(BooksExampleHomePage.SAVE_COMMAND))
            {
                String sBookId = req.getParameter("bookid");
                String sTitle = req.getParameter("title");
                String sAuthor = req.getParameter("author");
                String sNumPages = req.getParameter("numpages");

                if (sTitle.equals("") || sAuthor.equals("") ||
                                         sNumPages.equals(""))
                {
                    String sMessage = "The following fields are required:" +
                                    " Title, Author, NumPages";
                    MessagePage message =
                      new MessagePage(sServletAddress, "Error", sMessage);
                    out.print(message.getHtml());
                    return;
                }

                try
                {
                    int iNumPages = Integer.parseInt(sNumPages);
                } catch (NumberFormatException nfe)
                  {
                    String sMessage = "Num Pages must be an integer. " +
                                    "Click back to correct the error.";
                    MessagePage message =
                      new MessagePage(sServletAddress, "Error", sMessage);
                    out.print(message.getHtml());
                    return;
                  }

                String sql = null;
                sBookId = sBookId.trim();
                if (sBookId.equals("0") || sBookId.equals(""))
                {
                    String sQuery = "SELECT MAX(BookId) AS " +
                                    "Max_BookId FROM Books";
                    String sNewBookId = "1";
                    try
                    {
                        ResultSet rslt = stmt.executeQuery(sQuery);
                        if (rslt.next())
                            sNewBookId = "" + (rslt.getInt("Max_BookId") + 1);
                    } catch (SQLException sqle)
                      {
                        String sMessage = "A database error has occurred." +
                                          " Could not get new BookId";
                        MessagePage message =
                            new MessagePage(sServletAddress, "Error",
                                            sMessage + "<BR>Error: " +
                                            sqle.toString());
                        out.print(message.getHtml());
                        logSQLException(sqle);
                      }

                    // new record
                    sql = "INSERT INTO Books VALUES (" +
                                                sNewBookId + ", '" +
                                                sTitle + "', '" +
                                                sAuthor + "', " +
                                                sNumPages + ")";
                }
                else
                {
                    // existing record
                    sql = "UPDATE Books SET Title='" + sTitle + "', " +
                          "Author='" + sAuthor + "', NumPages=" + sNumPages +
                          " WHERE BookId=" + sBookId;
                }

                try
                {
                    int iStatus = stmt.executeUpdate(sql);
                    MessagePage message =
                        new MessagePage(sServletAddress, "Success",
                                     "Successfully saved the title.");
                    out.print(message.getHtml());
                } catch (SQLException sqle)
                  {
                    String sMessage = "A database error has occurred.";
                    MessagePage message =
                        new MessagePage(sServletAddress, "Error",
                                        sMessage + "<BR>Error: " +
                                        sqle.toString());
                    out.print(message.getHtml());
                    logSQLException(sqle);
                  }
            }
            else if (sCommand.equals(BooksExampleHomePage.DELETE_COMMAND))
            {
                String sBookId = req.getParameter("bookid");
                String sql = "DELETE FROM Books WHERE bookid=" + sBookId;

                try
                {
                    int iStatus = stmt.executeUpdate(sql);
                    MessagePage message =
                        new MessagePage(sServletAddress, "Success",
                                     "Successfully deleted the title.");
                    out.print(message.getHtml());
                } catch (SQLException sqle)
                  {
                    String sMessage = "A database error has occurred.";
                    MessagePage message =
                        new MessagePage(sServletAddress, "Error",
                                        sMessage + "<BR>Error: " +
                                        sqle.toString());
                    out.print(message.getHtml());
                    logSQLException(sqle);
                  }
            }
        }

    }

    private HtmlPage handleSearch(String sql, String sServletAddress)
    {
        try
        {
            ResultSet rslt = stmt.executeQuery(sql);
            Vector vRecords = new Vector();

            // Process data
            while (rslt.next())
            {
                BookRecord record = new BookRecord();
                record.setBookId(rslt.getInt("BookId"));
                record.setTitle(rslt.getString("Title"));
                record.setAuthor(rslt.getString("Author"));
                record.setNumPages(rslt.getInt("NumPages"));

                vRecords.addElement(record);
            }

            BookRecord records[] = null;
            if (vRecords.size() > 0)
            {
                records = new BookRecord[vRecords.size()];
                for (int iCount = 0; iCount < records.length; iCount++)
                    records[iCount] =
                            (BookRecord)vRecords.elementAt(iCount);
            }

            SearchResultsPage resultsPage =
                    new SearchResultsPage(sServletAddress,
                                          records,
                                          "all titles");
            rslt.close();
            return(resultsPage);

        } catch (SQLException sqle)
          {
            String sMessage = "A database error has occurred.";
            MessagePage message =
                new MessagePage(sServletAddress, "Error",
                                sMessage + "<BR>Error: " +
                                sqle.toString());
            logSQLException(sqle);
            return (message);
          }
    }

    public String getServletInfo() {
        return "Database servlet";
    }

}
