package jwiley.chp19;

import java.applet.*;		// For applet
import java.awt.*;			// for layout etc
import java.net.*;			// for URL
import java.io.IOException;	// for IO Exceptions
import javax.media.*;		// for the Java media class

/**
 *  Player3.java
 *  A class that creates a set of media player each playing in a infinite loop.
 *  sort of like row, row, row your boat only each runs at a different speed
 *  with the first running at normal or 1/1 time.  The second running at 1/2 time
 *  etc.  A start time could have been set with setMediaTime instead to make all
 *  the players play in sync but starting at different intervals.
 *
 *  @version 1.0
 *  @author A.J. Saganich Jr.
 */
public class Player3 
	extends Applet 
	implements ControllerListener
	//  requires controllerUpdate(ControllerEvent event) be implemented
{
	private static final 
	int 		ctPlayers 	= 3;
	Player 		players[] 	= null;
	Panel		playerPanel	= null;
	Component 	vc 			= null;
	Component	controls	= null;
	boolean 	fRunning 	= false;		// Not initially running

	/**
	 * Read the name of the media file and then begin processing
	 */
	public void init()
	{
		String inputFile = null;
		URL inputURL	 = null;

		/**
		 * the inputFile parameter contains the name
		 * of the media file to play.
		 */
		inputFile = getParameter("inputFile");
		if ( null == inputFile)
		{
			System.out.println("Invalid media file parameter");
			return;
		}

		System.out.println("Running on file: "+ inputFile);
		// Create an url from the file name and the url to the
		// document containing this applet.
		try
		{
			inputURL = new URL(getDocumentBase(), inputFile);
		}
		catch (MalformedURLException e)
		{
			System.out.println("Invalid media file URL!");
		}

		// Create the players
		try
		{
			players = new Player[ctPlayers];
			// Create a player that corresponds to the input type.
			for (int i = 0; i < ctPlayers; i++)
			{
				try
				{
					players[i] = Manager.createPlayer(inputURL);
				}
				catch (javax.media.NoPlayerException e)
				{
					players[i] = null;
				}
				if(players[i] != null)
				{
					players[i].addControllerListener(this);
				}
				else
				{
					System.out.println("Could not create player for " + inputURL);
				}
			}

			// build a panel for the clips
			playerPanel = new Panel();
			playerPanel.setLayout(new GridLayout(ctPlayers,1)); // n x 1 grid( a stack)

			
			// Use a border based layout for the whole thing
			setLayout(new BorderLayout());
			add ("Center",playerPanel);



		}
		catch(IOException e)
		{
			System.out.println("IO exception creating aPlayer for" + inputURL);
		}
	}



	/**
	 * Start method
	 * Note that this method attempts to start the prefetch of the
	 * clip but does not start the player playing.  When the prefetch
	 * completes the a event will be generated and the controllerUpdate
	 * method called which will start the clip playing.
	 */
	public void start()
	{
		for ( int i = 0; i < ctPlayers; i++)
		{
			if ( players[i] != null)
				 players[i].prefetch();
		}
	}

	/**
	 * Stop method
	 * stop the player playing, if allocated, and release its resources.
	 */
	public void stop()
	{
		fRunning = false;
		for ( int i = 0; i < ctPlayers; i++)
		{
			if (players[i] != null)
			{
				players[i].stop();
				players[i].deallocate();
			}
		}
	}

	/**
	 * controllerUpdate method
	 * required by the ControlllerLister interface
	 * called whenever a media event occurs
	 */
	public synchronized void controllerUpdate(ControllerEvent event)
	{

		Player which = (Player)event.getSource();
		int iWhich;
		for (iWhich = 0; iWhich < ctPlayers; iWhich++)
			if ( which == players[iWhich]) break;

		System.out.println("ControllerUpdate with event " + event.toString() + " for " + iWhich);

		// When the aPlayer is Realized, get the visual component
		// and add it to the Applet
		if (event instanceof RealizeCompleteEvent)
		{
			// force the applet to draw the image and the controls

			playerPanel.add(which.getVisualComponent());

			// first player goes normal speed (1/1)
			// second goes half speed (1/2)
			// thrid goes 1/4 speed etc
			which.setRate((float)(1.0/(iWhich+1.0)));
			if ( iWhich == 0)
			{	
				// The first is the master and all its controls are in terms of it.
				controls = players[0].getControlPanelComponent();
				if ( controls != null)
				{	
					controls.setBackground(Color.lightGray);
					add("North", controls);
				}
			}

			validate();
		}
		else if (event instanceof PrefetchCompleteEvent)
		{
			//
			//Are they all complete?
			//
			int i;
			for ( i = 0 ; i < ctPlayers; i++)
			{
				if (players[i].getState() != Player.Prefetched )
					break;
			}
			if ( i == ctPlayers) 
			{
				for ( i = 0 ; i < ctPlayers; i++)
				{
					players[i].start();
				}
			}
		}
		else if (event instanceof EndOfMediaEvent)
		{
			which.setMediaTime(new javax.media.Time(0));
			which.start();
		}
		else if ( event instanceof StartEvent)
		{
			if ( which == players[0])
			{
				// Note that we skip the first its already started.
				for ( int i = 0+1 ; i < ctPlayers; i++)
				{
					players[i].start();
				}
			}
		}
		else if ( event instanceof StopByRequestEvent)
		{
			// Note that we skip the first, don't want to get into an infinite loop!
			if ( which == players[0])
			{
				for ( int i = 0+1 ; i < ctPlayers; i++)
				{
					players[i].stop();
				}
			}
		}
		else 
		{
			// Some other event occured
			// Adding fast forward and rewind events is left as an execise
			// to the reader.
		
		}
	} // End method controllerUpdate
} // End class Player3
