package jwiley.chp19;
import java.applet.*;		// For applet
import java.awt.*;			// for layout etc
import java.net.*;			// for URL
import java.io.IOException;	// for IO Exceptions
import javax.media.*;		// for the Java media class
import jwiley.chp3.Warning;		// for our Warning/error message class

/**
 *  TrivialPlayer.java
 *  A class that creates a trivial media player
     which plays a clip in an infinite loop
 *  @version 1.0
 *  @author A.J. Saganich Jr.
 */
public class TrivialPlayer 
	extends Applet 
	implements ControllerListener
	//  requires controllerUpdate(ControllerEvent event) be implemented
{
	Player 		aPlayer 	= null;  
	Component 	vc 			= null;
	boolean 	fRunning 	= false;		// Not initially running

	/**
	 * Read the name of the media file and then begin processing
	 */
	public void init()
	{
		String inputFile = null;
		URL inputURL	 = null;
		Warning msg 	= null; // Create one if we need it.

		// Use a border based layout
		setLayout(new BorderLayout());

		/**
		 * the inputFile parameter contains the name
		 * of the media file to play.
		 */
		inputFile = getParameter("inputFile");
		if ( null == inputFile)
		{
			System.out.println("Invalid media file parameter");
			msg = new Warning("inputFile","Must set value of inputFile in web page!");
			return;
		}

		System.out.println("Running on file: "+ inputFile);
		try
		{
			// Create an url from the file name and the url to the
			// document containing this applet.
			inputURL = new URL(getDocumentBase(), inputFile);
			// Create a player that corresponds to the input type.
			try
			{
				aPlayer = Manager.createPlayer(inputURL);
			}
			catch (javax.media.NoPlayerException e)
			{	
				aPlayer = null;
			}
			// If the player was created then we want 
			// to listen to its events.
			if(aPlayer != null)
			{
				aPlayer.addControllerListener(this);
			}
			else
			{
				System.out.println("Could not create aPlayer for " + inputURL);
				msg = new Warning("createPlayer","Could not create aPlayer on given input!");
			}
		}
		catch (MalformedURLException e)
		{
			System.out.println("Invalid media file URL!");
			msg = new Warning("MalformedURLException","Could not create aPlayer on given input!");
		}
		catch(IOException e)
		{
			System.out.println("IO exception creating aPlayer for" + inputURL);
			msg = new Warning("IOException","Could not create aPlayer on given input!");
		}
	}

	/**
	 * Start method
	 * Note that this method attempts to start the prefetch of the
	 * clip but does not start the player playing.  When the prefetch
	 * completes the a event will be generated and the controllerUpdate
	 * method called which will start the clip playing.
	 */
	public void start()
	{
		if (aPlayer != null)
			aPlayer.prefetch();
	}

	/**
	 * Stop method
	 * stop the player playing, if allocated, and release its resources.
	 */
	public void stop()
	{
		if (aPlayer != null)
		{
			fRunning = false;
			aPlayer.stop();
			aPlayer.deallocate();
		}
	}

	/**
	 * controllerUpdate method
	 * required by the ControlllerLister interface
	 * called whenever a media event occurs
	 */
	public synchronized void controllerUpdate(ControllerEvent event)
	{
		System.out.println("ControllerUpdate entered!");
		System.out.println("\twith event " + event.toString());
		// When the aPlayer is Realized, get the visual component
		// and add it to the Applet
		if (event instanceof RealizeCompleteEvent)
		{
			vc = aPlayer.getVisualComponent();
			if (vc != null)
			{
				vc.setBackground(Color.gray);
				add("Center", vc);
			}

			// force the applet to draw the component
			validate();
		}
		else if (event instanceof PrefetchCompleteEvent)
		{
			aPlayer.start();
			fRunning = true;
		}
		else if (event instanceof EndOfMediaEvent)
		{
			aPlayer.setMediaTime(new javax.media.Time(0));
			if ( true == fRunning)
				aPlayer.start();
		}
		else
		{
			// Some other event occured
		}
	} // End method controllerUpdate
} // End class TrivialPlayer
