// ReferenceDemo.java
package jwiley.chp2;

/**
 * Simple object for demonstration.
 * @see PointerEquiv
 */
class simpleObject
{
    /** an int. */
    int a;
    /** left and right references. */
    public simpleObject left, right;

    /** constructor. */
    simpleObject(int inA)
    {
        a=inA;
        left = right = null;
    }

    /** accessor. */
    int getA()
    { return a; }

    /** mutator. */
    void setA(int inA)
    { a = inA; }

    /** Create a String representation. */
    public String toString()
    {
        return new String("a: " + a);
    }
}

/**
 * Class to demonstrate Java references.
 * @author Michael C. Daconta
 * @version 1.2
 */
class ReferenceDemo
{
    /** Method to demonstrate pass by value. */
    static void swapRef(Object a, Object b)
    {
            Object tmp = a;
            a = b;
            b = tmp;
    }

    /** Method to demonstrate swapping object contents. */
    static void swapBody(simpleObject a, simpleObject b)
    {
        int aInt = a.getA();
        a.setA(b.getA());
        b.setA(aInt);
    }

    public static void main(String args[])
    {
        /* 1) Like pointers, all objects and arrays
              are dynamically allocated. */
        int arr[] = new int[10];
        simpleObject myObj = new simpleObject(5);

        /* 2) A Java reference is implemented
              via a handle (pointer pointer) that can point
              to another object. */
        int arr2[] = arr;
        simpleObject yourObj = myObj;
        System.out.println("yourObj is " + yourObj.toString());

        /* 3) null is a reserved keyword identical to
              NULL in usage. */
        int arr3[] = null;
        arr = null; /* causes arr and arr2 to point to null */

        /* 4) Arrays and Objects are placed on the stack as
              a handle (not pass by value).  However, there is
              no Pass by Reference in Java at this time. */
        simpleObject otherObject = new simpleObject(20);
        System.out.println("Before swapRef, myObj is " + myObj.toString() +
                           " otherObject is " + otherObject.toString());
        swapRef(myObj, otherObject);
        System.out.println("After swapRef, myObj is " + myObj.toString() +
                           " otherObject is " + otherObject.toString());

        /* 5) Since you are passing object references, you can
              modify object data members. */
        System.out.println("Before swapBody, myObj is " + myObj.toString() +
                           " otherObject is " + otherObject.toString());
        swapBody(myObj, otherObject);
        System.out.println("After swapBody, myObj is " + myObj.toString() +
                           " otherObject is " + otherObject.toString());


        /* 6) There is no explicit free or delete.  Java
              memory is garbage collected. */
        System.gc();  /* Request garbage collection to occur NOW. */

        /* 7) Since all Java objects are descendants of a
              single Object class.  An unitialized Object
              can act like a void pointer. */
        Object voidp = otherObject;
        ((simpleObject) voidp).setA(100);
        System.out.println("voidp points to " +
                            ((simpleObject) voidp).toString());
        System.out.println("Just as otherObject now is " +
                                otherObject.toString());

        /* 7) Self-referential structures are possible by
              using non-instantiated objects. */
        simpleObject root = new simpleObject(1000);
        root.left = new simpleObject(500);
        root.right = new simpleObject(2000);
    }
}
