/** DatabaseTableModel.java */
package jwiley.chp21;

import java.sql.*;
import java.util.Vector;
import javax.swing.*;
import javax.swing.table.*;

/**
 * DataModel for the JTable component.
 * @author Eric Monk
 * @author Mike Daconta
 */
public class DatabaseTableModel extends DefaultTableModel
{
    /** Constructor to build data for JTable by querying any jdbc
     *  table.
     * @param sDatabaseURL - url connection to database in jdbc format.
     * @param sTable - table name to query.
     */
    public DatabaseTableModel (String sDatabaseURL, String sTable) throws Exception
    {
        try
        {
            // Load database driver
            Class.forName ("sun.jdbc.odbc.JdbcOdbcDriver");
        } catch (ClassNotFoundException cnfe)
          {
            throw new Exception("Error initializing database driver. " +
                                "Error: " + cnfe.toString());
          }

        // getMetaData - column headings
        // get all data from table

        try
        {
            // Establish connection
            Connection con = DriverManager.getConnection(sDatabaseURL);

            // Create and send statement
            String sQuery = "SELECT * FROM " + sTable;

            Statement stmt = con.createStatement();
            ResultSet rslt = stmt.executeQuery(sQuery);
            ResultSetMetaData rsmd = rslt.getMetaData();

            int iNumColumns = rsmd.getColumnCount();
            String [] sColumnHeadings = new String[iNumColumns];

            for (int iCount = 0; iCount < sColumnHeadings.length; iCount++)
                sColumnHeadings[iCount] = rsmd.getColumnName(iCount + 1);

            setColumnIdentifiers(sColumnHeadings);

            // Process data
            while (rslt.next())
            {
                String sDataForRow[] = new String[sColumnHeadings.length];

                for (int iCount = 0; iCount < sColumnHeadings.length; iCount++)
                {
                    sDataForRow[iCount] = rslt.getString(iCount + 1);
                }

                addRow(sDataForRow);
            }

            // Cleanup
            rslt.close();
            stmt.close();
            con.close();

        } catch (SQLException sqle)
          {
            while (sqle != null)
            {
                System.out.println ("SQLState: " +
                                    sqle.getSQLState ());
                System.out.println ("Message:  " +
                                    sqle.getMessage ());
                System.out.println ("ErrorCode:   " +
                                    sqle.getErrorCode ());
                sqle = sqle.getNextException ();
            }
            throw new Exception ("Database error has occurred. " +
                                 "Error: " + sqle.toString());

          }
    }
}