/** SimplePanel.java */
package jwiley.chp21;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * An application that displays a JButton and two JRadioButtons.
 * The JRadioButtons determine the look and feel used by the application.
 */
public class SimplePanel extends JPanel {
    static JFrame frame;
    static String nativeLf = "Native";
    static String javaLf = "Java";

    /** Constructor. */
    public SimplePanel()
	{
	    super(true);

    	// start out with Java look and feel
        try
        {
            UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
        } catch (Exception e)
          {
            System.err.println("Could not load Java look and Feel.");
          }

        setLayout(new BorderLayout());
	    setBackground(Color.lightGray);

    	// Create a button.
    	JButton button = new JButton("Hello, world");
        button.setMnemonic('h');

        // Create the glyphs for radio buttons
        ImageIcon radio = new ImageIcon("images\\offbut.jpg");
        ImageIcon radioSelected = new ImageIcon("images\\onbut.jpg");

        // Create some radio buttons with image glyphs
        JRadioButton testButton = new JRadioButton("Radio1");
        testButton.setSelected(true);
        JRadioButton test2Button = new JRadioButton("Radio2");
        ButtonGroup testGroup = new ButtonGroup();
        testGroup.add(testButton);
        testGroup.add(test2Button);

    	JRadioButton nativeButton = new JRadioButton(nativeLf, radio);
        nativeButton.setMnemonic('n');
    	nativeButton.setActionCommand(nativeLf);
    	nativeButton.setSelected(true);
    	nativeButton.setPressedIcon(radioSelected);

    	JRadioButton javaButton = new JRadioButton(javaLf, radio);
        javaButton.setMnemonic('j');
    	javaButton.setActionCommand(javaLf);
    	javaButton.setPressedIcon(radioSelected);

    	// Group the radio buttons.
    	ButtonGroup group = new ButtonGroup();
    	group.add(nativeButton);
    	group.add(javaButton);

        // Register a listener for the radio buttons.
    	RadioListener myListener = new RadioListener();
    	javaButton.addActionListener(myListener);
    	nativeButton.addActionListener(myListener);

        JPanel centerP = new JPanel();
        centerP.setLayout(new FlowLayout(FlowLayout.LEFT,5,5));
        JPanel southP = new JPanel();

    	centerP.add(button);
    	centerP.add(testButton);
    	centerP.add(test2Button);

    	// controls
    	southP.add(nativeButton);
    	southP.add(javaButton);

        // add Panels
        add("Center", centerP);
        add("South", southP);
    }


    /** An ActionListener that listens to the radio buttons. */
    class RadioListener implements ActionListener
    {
	    public void actionPerformed(ActionEvent evt)
	    {
	        String lfName = null;

	        if (evt.getActionCommand().equals(javaLf))
	            lfName = UIManager.getCrossPlatformLookAndFeelClassName();
	        else
	            lfName = UIManager.getSystemLookAndFeelClassName();

            try
            {
                UIManager.setLookAndFeel(lfName);
		        SwingUtilities.updateComponentTreeUI(frame);
            } catch (Exception e)
              {
                    System.err.println("Could not load and feel: "
				                        + lfName);
              }
	    }
    }

    /** main() method to invoke from JVM. */
    public static void main(String args[])
    {
         WindowListener l = new WindowAdapter()
         {
             public void windowClosing(WindowEvent e)
             {System.exit(0);}
         };

         frame = new JFrame("Simple Example");
         frame.getContentPane().setLayout(new BorderLayout(5,5));
         frame.addWindowListener(l);
         frame.getContentPane().add("Center", new SimplePanel());
         frame.pack();
         frame.setLocation(50,50);
         frame.setVisible(true);
    }
}
