/** TocFrame.java */
package jwiley.chp21;

import javax.swing.*;
import javax.swing.tree.*;

import java.awt.*;
import java.awt.event.*;
import java.io.IOException;

/**
 * Simple Frame to demonstrate a dynamic use for a
 * JTree.
 * @author Michael C. daconta
 * @see TocTreePanel
 */
public class TocFrame extends Frame
{
    /** Simple adapter class to shutdown the application
        when the close button is pushed. */
    class ShutdownAdapter extends WindowAdapter
    {
        public void windowClosing(WindowEvent we)
        { System.exit(0); }
    }

    /** Constructor.
     * @param sTocFile Name of ".toc" file.
     */
    public TocFrame(String sTocFile)
    {
        super(sTocFile);
        TocTreePanel tp = null;
        try
        {
            tp = new TocTreePanel(sTocFile);
            add("Center", tp);
        } catch(IOException ioe)
          { }

        addWindowListener(new ShutdownAdapter());

        setSize(200,200);
        setLocation(50,50);
        setVisible(true);
    }

    /** Main() method to invoke from the JVM. */
    public static void main(String args[])
    {
        if (args.length < 1)
        {
            System.out.println("USAGE: java TocFrame tocFile");
            System.exit(1);
        }

        new TocFrame(args[0]);
    }
}
