// ClassInspector.java
package jwiley.chp6;

import java.lang.reflect.*;

/**
 * Class to enumerate all fields, methods and constructors of a Java class.
 * The output of this class is similar to the JDK utility javap
 */
public class ClassInspector
{
    /**
     * Translates an array of exceptions into a throws clause
     */
    public static void PrintExceptions(Class exceptions[])
    {
        if (exceptions.length != 0)
            System.out.print(" throws ");

        for (int i = 0; i < exceptions.length; i++)
        {
            if (i != 0)
            {
                System.out.print(", ");
            }
            System.out.print(exceptions[i].getName());
        }

        System.out.println();
    }

    /**
     * Converts an array of classes into a parameter list for a function
     */
    public static void PrintParameters(Class params[])
    {
        System.out.print("(");
        for (int i = 0; i < params.length; i++)
        {
            if (i != 0)
            {
                System.out.print(", ");
            }
            System.out.print(params[i].getName());
        }
        System.out.print(")");
    }

    /**
     * Prints all fields for a class as they would
     * appear in a java source file
     */
    public static void PrintFields(Field fields[])
    {
        System.out.println("Fields:");
        if (fields.length == 0)
        {
            System.out.println("\tNo public fields");
        } else {
            for (int i = 0; i < fields.length; i++)
            {
                System.out.println("\t" + 
Modifier.toString(fields[i].getModifiers()) + " " +
fields[i].getType().getName() + " " +
fields[i].getName());
            }
        }
    }

    /**
     * Prints the signatures of all methods in a class as they
     * would be displayed by javap
     */
    public static void PrintMethods(Method methods[])
    {
        System.out.println("Methods:");
        if (methods.length == 0)
        {
            System.out.println("\tNo public methods");
        } else {
            for (int i = 0; i < methods.length; i++)
            {
                System.out.print("\t" +
                       Modifier.toString(methods[i].getModifiers()) +
                       " " + methods[i].getReturnType().getName() + " " +
                       methods[i].getName());
                PrintParameters(methods[i].getParameterTypes());
                PrintExceptions(methods[i].getExceptionTypes());
            }
        }
    }

    /**
     * Prints the signatures of all constructors in a class as they
     * would be displayed by javap
     */
    public static void PrintConstructors(Constructor constructors[])
    {
        System.out.println("Constructors:");
        if (constructors.length == 0)
        {
            System.out.println("\tNo public constructors");
        } else {
            for (int i = 0; i < constructors.length; i++)
            {
                System.out.print("\t" +
                       Modifier.toString(constructors[i].getModifiers()) +
                       " " + constructors[i].getName());
                PrintParameters(constructors[i].getParameterTypes());
                PrintExceptions(constructors[i].getExceptionTypes());
            }
        }
    }

    /**
     * Examines the structure of a class and prints
     * the structure to the console
     */
    public static void Inspect(Class c)
    {
        System.out.println("Contents of class " + c.getName());
        PrintFields(c.getFields());
        PrintConstructors(c.getConstructors());
        PrintMethods(c.getMethods());
    }

    /**
     * Allows users to invoke the ClassInspector from the command
     * line.  This routine expects the name of the class to inspect
     * to be entered as a command line argument
     */
    public static void main(String[] argv) throws ClassNotFoundException
    {
        if (argv.length == 0)
        {
            System.out.println("USAGE: java ClassInspector <class name>");
            return;
        }

        Class c = Class.forName(argv[0]);
        Inspect(c);
    }
}
