
package jwiley.chp7;

import java.awt.*;

public class SimplePositioner extends Canvas
{
    Point       pMarkerPosition = new Point(0,0);
    Rectangle   rectMarkerSize;
    Color       colorBack = Color.white;
    Color       colorMarker = Color.black;

    Graphics    offScreenGraphics;
    Image       offScreenImage;

    Point       pKeepMarkerPosition = new Point(0,0);
    boolean     bMarkerVisible = false;

    public SimplePositioner (Point pInitPosition, Rectangle rectMarkerSize)
    {
        this.rectMarkerSize = rectMarkerSize;
        setBackground(colorBack);
        setForeground(colorMarker);

        setMarker(pInitPosition.x, pInitPosition.y);
    }

    public void paint (Graphics g)
    {
        if (offScreenImage != null)
        {
            g.drawImage(offScreenImage, 0,0,this);
        }
        else
            createOffScreenGraphics();
    }

    public void update (Graphics g)
    {
        if (offScreenImage != null)
        {
            drawMarker(offScreenGraphics);
        }
        paint(g);
    }

    private void drawMarker (Graphics g)
    {
        // first time through initial marker is not
        //  visible so don't XOR to erase
        if (bMarkerVisible)
        {
            g.setColor(colorBack);
            draw(pKeepMarkerPosition, g);
        }
        g.setColor(colorMarker);
        draw(pMarkerPosition, g);
        bMarkerVisible = true;

        pKeepMarkerPosition.x = pMarkerPosition.x;
        pKeepMarkerPosition.y = pMarkerPosition.y;
    }

    public void createOffScreenGraphics()
    {
        Dimension d = getSize();
        offScreenImage = createImage(d.width, d.height);
        if (offScreenImage != null)
        {
            offScreenGraphics = offScreenImage.getGraphics();
            repaint();
        }
    }

    public void draw (Point pPosition, Graphics g)
    {
        Point pCoor = translatePercentToCoordinates(pPosition);

        int iXPosition = pCoor.x - (int)(rectMarkerSize.width / 2);
        int iYPosition = pCoor.y - (int)(rectMarkerSize.height / 2);
        g.fillRect(iXPosition, iYPosition, rectMarkerSize.width, rectMarkerSize.height);
    }

    // translates a Point containing percentages to actual pixel values in the canvas
    private Point translatePercentToCoordinates (Point pPercent)
    {
        int iXCoor = 0;
        int iYCoor = 0;

        iXCoor = (int)(this.getSize().width * pPercent.x * .01);
        iYCoor = (int)(this.getSize().height * pPercent.y * .01);

        return (new Point(iXCoor, iYCoor));
    }


    public void setSize(Dimension d)
    {
        setSize(d.width, d.height);
    }

    public void setSize(int x, int y)
    {
        super.setSize(x, y);
        createOffScreenGraphics();
    }

    public void setMarker (int iXPercent, int iYPercent)
    {
        pMarkerPosition = enforcePercent(new Point(iXPercent, iYPercent));
        repaint();
    }

    private Point enforcePercent (Point pPercent)
    {
        int iXCoor = 0;
        int iYCoor = 0;

        iXCoor = pPercent.x;
        iYCoor = pPercent.y;

        if (iXCoor < 0)
            iXCoor = 0;
        else if (iXCoor > 100)
            iXCoor = 100;

        if (iYCoor < 0)
            iYCoor = 0;
        else if (iYCoor > 100)
            iYCoor = 100;

        return (new Point(iXCoor, iYCoor));
    }


    public static void main (String args[])
    {
        Frame f = new Frame();
        SimplePositioner sp = new SimplePositioner(new Point(50,50), new Rectangle(6,6));
        f.setLayout(new BorderLayout());
        f.add("Center", sp);
        f.setSize(200,200);
        f.show();

        try {   Thread.sleep(2000);  } catch (Exception e)   {   }

        for (int iCount = 0; iCount < 100; iCount++)
        {
            try {   Thread.sleep(30);  } catch (Exception e)   {   }
            sp.setMarker(iCount, iCount);
        }

        for (int iCount = 100; iCount > 0; iCount--)
        {
            try {   Thread.sleep(30);  } catch (Exception e)   {   }
            sp.setMarker(iCount, 100 - iCount);
        }
    }
}
