
package jwiley.chp7;

import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import java.util.Enumeration;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;

public class ZipUtil
{
    public static final int BUFFER_SIZE = 2048;

    public static void main (String args[])
    {
        if (args.length < 2 || args.length > 3)
        {
            System.out.println("# Args: " + args.length);
            System.out.println("Invalid number of arguments!");
            printUsage();
            System.exit(0);
        }

        String sFileList = null;
        String sZipFile = null;

        if (args[0].equals("-l"))
        {
            listZipFiles(args[1]);
        }

        else if (args[0].equals("-d"))
        {
            if (args.length == 3)
            {
                makeZipFile(args[1], args[2]);
            }
            else
            {
                System.out.println("Invalid use of -d parameter.");
                printUsage();
                System.exit(0);
            }

        }

        else if (args[0].equals("-i"))
        {
            unpackZipFile(args[1]);
        }

        else
        {
            System.out.println("Invalid argument: " + args[0]);
            printUsage();
            System.exit(0);
        }
    }

    static void listZipFiles(String sZipFile)
    {
        try
        {
            ZipFile zipFile = new ZipFile(sZipFile);
            Enumeration enumZipFile = zipFile.entries();

            while(enumZipFile.hasMoreElements())
            {
                ZipEntry zipEntry = (ZipEntry) enumZipFile.nextElement();

                System.out.println(zipEntry.getName() + " is " +
                                   zipEntry.getSize() + " bytes.");
            }

            zipFile.close();

        } catch (IOException ioe)
          {
            System.out.println("Error with zip file: " + sZipFile +
                                    ".\nError: " + ioe.toString());
          }
    }

    static void makeZipFile(String sFileList, String sZipFile)
    {
        try
        {
            File fCurrentDir = new File(".");
            String sFiles[] = fCurrentDir.list();

            FileOutputStream fosZipFile = new FileOutputStream(sZipFile);
            ZipOutputStream zosZipFile = new ZipOutputStream(fosZipFile);

            byte buffer[] = new byte[BUFFER_SIZE];

            for (int iCount = 0; iCount < sFiles.length; iCount++)
            {
                if (fileMatches(sFiles[iCount], sFileList))
                {
                    try
                    {
                        // add to zip file
                        System.out.println("Adding " +
                            sFiles[iCount] + " to " + sZipFile + "...");
                        FileInputStream fisFileToAdd =
                                    new FileInputStream(sFiles[iCount]);

                        ZipEntry zipEntry = new ZipEntry(sFiles[iCount]);
                        zosZipFile.putNextEntry(zipEntry);

                        int iNumBytesRead = 0;
                        while (true)
                        {
                            iNumBytesRead = fisFileToAdd.read(buffer);
                            if (iNumBytesRead == -1)
                                break;

                            zosZipFile.write(buffer, 0, iNumBytesRead);
                        }

                        fisFileToAdd.close();
                        zosZipFile.closeEntry();

                    } catch (IOException ioe1)
                      {
                        System.out.println("Error adding " +
                            sFiles[iCount] + " to zip file.\nError: " +
                            ioe1.toString());
                      }
                }
            }

            zosZipFile.close();

        } catch (IOException ioe2)
          {
            System.out.println("Error making zip file.\nError: " +
                                                    ioe2.toString());
          }


    }

    static void unpackZipFile(String sZipFile)
    {
        try
        {
            FileInputStream fisZipFile = new FileInputStream(sZipFile);
            ZipInputStream zisZipFile = new ZipInputStream(fisZipFile);

            byte buffer[] = new byte[BUFFER_SIZE];

            while (true)
            {
                ZipEntry zipEntry = zisZipFile.getNextEntry();
                if (zipEntry == null)
                    break;

                try
                {
                    // get from zip file
                    System.out.println("Extracting " + zipEntry.getName() +
                                                " from " + sZipFile + "...");
                    FileOutputStream fosFileToExtract =
                                new FileOutputStream(zipEntry.getName());

                    int iNumBytesRead = 0;
                    while (true)
                    {
                        iNumBytesRead = zisZipFile.read(buffer,
                                                    0, buffer.length);
                        if (iNumBytesRead == -1)
                            break;

                        fosFileToExtract.write(buffer, 0, iNumBytesRead);
                    }

                    fosFileToExtract.close();
                    zisZipFile.closeEntry();

                } catch (IOException ioe)
                  {
                    System.out.println("Error extracting " +
                           zipEntry.getName() + " from zip file." +
                           "\nError: " + ioe.toString());
                  }
            }

            zisZipFile.close();

        } catch (IOException ioe2)
          {
            System.out.println("Error making zip file.\nError: " +
                                                        ioe2.toString());
          }
    }

    static boolean fileMatches(String sFileName, String sWildcardPattern)
    {
        // currently only handle asterisks at beginning of string
        if (sWildcardPattern.equals("*"))
            return (true);
        else
        {
            // get last part of WildcardPattern
            String sExtension = sWildcardPattern.substring(1);
            if (sFileName.endsWith(sExtension))
                return (true);
        }

        return (false);
    }

    static void printUsage()
    {
        System.out.println();
        System.out.println("Usage: java ZipUtil [-i | -d | -l]");
        System.out.println("\tinflate\t-i <zip file>");
        System.out.println("\tdeflate\t-d <file(s)> <zip file>");
        System.out.println("\tlist\t-l <zip file>");
    }
}



