// Copyright (c) 1995 by Michael C. Daconta
package jwiley.chp8;

import java.io.RandomAccessFile;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.DataInputStream;
import java.io.IOException;
import java.io.File;
import java.io.PrintStream;
import java.util.Vector;
import jwiley.chp8.database.*;

/** Main program that allows the creation of simple,
    single-table databases. */
class EZDB
{
    /** buffered reader to receive input. */
    static BufferedReader br = new BufferedReader(
                               new InputStreamReader(System.in));

    /** method to select which db to operate on. */
    static DbHeader select_db()
    {
        DbHeader outHeader=null;
        System.out.print("Enter the database name: ");
        System.out.flush();
        String buf=null;
        try {
                buf = br.readLine();
        } catch (Exception e) { return null; }

        if (buf.length() > 0)
        {
            buf.replace(' ', '_');

            // check if file exists
            File afile = new File(buf + ".schema");
            if (afile.exists())
                outHeader = new DbHeader(buf + ".schema");
            else
                return null;
        }
        else
            System.out.println("Invalid db name.");

        System.out.println(buf + " selected...OK");
        return outHeader;
    }

    /** main() method to invoke from JVM. */
    public static void main(String args[])
    {
        boolean done = false;
        DB theDB = null;
        Schema DBschema = null;

        while (!done)
        {
            PrintStream out = System.out;   // save typing space
            out.println(" ");
            out.println("<<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>");
            out.println("<<<<<<<<<<<< EZ DB 1.02J >>>>>>>>>>>>>>>");
            out.println("<<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>");
            out.println(" ");
            out.println("          0) Philosophy & Credits");
            out.println("          1) Define DB");
            out.println("          2) Select DB");
            out.println("          3) Add Record");
            out.println("          4) View Records");
            out.println("          5) Close DB");
            out.println("          6) Exit");
            out.println(" ");
            out.print("Enter choice : ");
            out.flush();
            String choiceStr=null;
            try {
              choiceStr = br.readLine();
            } catch (IOException ioe)
              {
                System.out.println("Error reading choice.");
                System.exit(1);
              }

            int choice = -1;
            if (choiceStr.length() > 0)
                choice = Integer.parseInt(choiceStr);

            switch (choice)
            {
              case 0:
                // philosophy & credits
                out.println(" ");
                out.println("<<<<<<<<<<< EZDB Philosophy >>>>>>>>>>");
                out.println("This program is based on the idea that");
                out.println("most people do not need an expensive, ");
                out.println("feature-packed database. What the common ");
                out.println("person needs is a simple, efficient ");
                out.println("database to help them keep track of the");
                out.println("people, places, and things in their");
                out.println("lives.  EZDB is a free solution. Enjoy!");
                out.println(" ");

                out.println("The Original EZDB was in C and had more");
                out.println("options (i.e. find, delete, modify...).");
                out.println("There was also a version with a GUI.");
                out.println("This is just a demonstration of the");
                out.println("concepts in Java.");
                out.println(" ");

                out.println("Press Enter to continue: ");
                    try { br.readLine(); } catch (Exception e) { }
                    break;
              case 1: // define DB
                DBschema = new Schema();
                if (DBschema.buildSchema())
                {
                    DBschema.outputSchema();
                    System.out.println("Schema stored as : " +
                                            DBschema.getFileName());
                }
                break;
              case 2: // select DB
                if (theDB == null)
                {
                   DbHeader aHeader = select_db();
                   if (aHeader != null)
                   {
                        theDB = new DB();
                        theDB.setHeader(aHeader);
                   }
                   else
                        System.out.println("DB does not exist.");
                }
                else
                   System.out.println("Close current DB first.  Choose Option 5.");
                break;
              case 3: // add record
                if (theDB != null)
                {
                    System.out.println("Enter Record data...");
                    Vector recVals = theDB.createRecord();

                    if (!theDB.addRecord(recVals))
                        System.out.println("Unable to add Record to DB.");
                    else
                        System.out.println("Successfully added Record to DB.");
                }
                else
                    System.out.println("MUST selecte a DB.  Choose option 2.");
                break;
              case 4: // view records
                if (theDB != null)
                    theDB.viewRecords();
                else
                    System.out.println("MUST selecte a DB.  Choose option 2.");

                break;
              case 5: // close DB
                if (theDB != null)
                {
                    theDB.close();
                    theDB = null;
                }
                break;
              case 6: // exit
                done = true;
                if (theDB != null)
                    theDB.close();
                break;
              default:
                System.out.println("Choice must be between 0 and 10.");
            }
        }
    }
}
