// LineNumFilter.java
package jwiley.chp8;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.LineNumberReader;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.FileNotFoundException;

/** Class to demonstrate the LineNumberReader class as a filter. */
public class LineNumFilter extends LineNumberReader
{
    /** text to add to stream. */
    private static String lineNumber=null;
    /** Current position within string version of line number. */
    private static int StrPos=0;
    /** Current character in stream. */
    private char currentChar;

    /** constructor. */
    LineNumFilter(InputStreamReader src)
    {
        super(src);
        lineNumber = new String((getLineNumber()+1) + " ");
    }

    /** override the read method of the stream to act as
        a filter. */
    public int read()
    {
        int input=0;

        if (lineNumber != null && lineNumber.length() > 0
            && StrPos < lineNumber.length())
        {
            currentChar = lineNumber.charAt(StrPos++);

            if (StrPos == lineNumber.length())
            {
                // dumped line number - reset
                StrPos = 0;
                lineNumber = null;
            }
        }
        else
        {
            try
            {
                input = super.read();
            } catch (IOException ioe) { return -1; }

            if (input == -1)
                    return input;
            currentChar = (char) input;
            if (currentChar == '\n')
            {
                // set up lineNumber
                lineNumber =
                    new String((getLineNumber()+1) + " ");
            }
        }

        return currentChar;
    }

    /** main() method to invoke from JVM. */
    public static void main(String args[])
    {
        if (args.length < 1)
        {
            System.out.println("USAGE: java runLineNumFilter fileToRead [-output lineFile]");
            System.out.println("  -output flag is optional.");
            System.exit(1);
        }

        String fileName = null;
        String lineFileName = null;
        boolean captureLines = false;
        for (int i=0; i < args.length; i++)
        {
            if (args[i].equals("-output"))
            {
                if (args.length > i + 1)
                {
                    lineFileName = new String(args[i+1]);
                    i++;
                    captureLines = true;
                }
                else
                {
                    System.out.println("No output file specified.");
                    System.exit(1);
                }
            }
            else
                fileName = new String(args[i]);
        }

        System.out.println("fileName is " + fileName);

        FileInputStream fis=null;
        // open the input file
        try {
            fis = new FileInputStream(fileName);
        } catch (FileNotFoundException fnf)
          {
            System.out.println("Unable to open " + fileName);
            System.exit(1);
          }

        if (captureLines)
            System.out.println("Line File is " + lineFileName);

        LineNumFilter lf = new LineNumFilter(new InputStreamReader(fis));
        FileOutputStream fos = null;
        PrintWriter pw = null;

        if (captureLines)
        {
           try {
                fos = new FileOutputStream(lineFileName);
                pw = new PrintWriter(fos);
          } catch (IOException e)
            {
                System.out.println("IO Error on " + lineFileName);
                System.exit(1);
            }
            catch (Exception e)
            {
                System.out.println("Unable to filter input");
                System.exit(1);
            }

        }

        int in=0;
        while ( (in = lf.read()) != -1)
        {
            if (captureLines)
            {
                pw.print((char)in);
                pw.flush();
            }
            else
            {
                System.out.print((char)in);
                System.out.flush();
            }
        }

        try {
          fis.close();
          fos.close();
        } catch (Exception e) { }
    }
}
