
package jwiley.chp9;

import java.io.DataOutputStream;
import java.io.DataInputStream;
import java.io.IOException;

import java.net.Socket;

public class ChatClient implements IChatClient, IMessageHandler
{
    Socket          sockClient;
    IChatDisplay    display;

    DataOutputStream    sendChannel;
    ClientProxy         clientProxy;

    String          sUserName;
    String          sHostName;
    int             iPort;

    public ChatClient (String sHostName, int iPort)
    {
        this.sHostName = sHostName;
        this.iPort = iPort;
    }

    public void connect (String sUserName)
    {
        try
        {
            sockClient = new Socket(sHostName, iPort);

            DataInputStream dis = new DataInputStream(sockClient.getInputStream());
            DataOutputStream dos = new DataOutputStream(sockClient.getOutputStream());

            this.sUserName = sUserName;
            dos.writeUTF(sUserName);

            sendChannel = dos;
            clientProxy = new ClientProxy(this, dis);
        } catch (IOException ioe)
          {
            display("Error connecting to chat server. " +
                        ioe.toString());
          }
    }

    public void disconnect ()
    {
        sendMessage("DISCONNECT", sUserName);
        display("Bye.");

        try
        {
            clientProxy.stopListen();
            clientProxy = null;

            sendChannel = null;

            sockClient.close();
            sockClient = null;
        } catch (IOException ioe) { }
    }

    public void setChatDisplay (IChatDisplay chatDisplay)
    {
        this.display = chatDisplay;
    }

    public void handleMessage (String sSpeaker, String sMessage)
    {
        if (sSpeaker.equals("SYS_SERVER"))
            display(">> " + sMessage);
        else
            display(sSpeaker + ": " + sMessage);
    }

    public void speak (String sMessage)
    {
        sendMessage(this.sUserName, sMessage);
    }

    private void sendMessage (String sSpeaker, String sMessage)
    {
        if (sendChannel != null)
        {
            try
            {
                sendChannel.writeUTF(sSpeaker);
                sendChannel.writeUTF(sMessage);
            } catch (IOException ioe)
              {
                display("Error sending message. " + ioe.toString());
              }
        }
    }

    private void display (String sMessage)
    {
        if (display == null)
            System.out.println(sMessage);
        else
            display.showMessage(sMessage);
    }
}