
package jwiley.chp9;

import java.io.DataOutputStream;
import java.io.DataInputStream;
import java.io.IOException;

import java.net.ServerSocket;
import java.net.Socket;

import java.util.Vector;

public class ChatServer extends Thread implements IMessageHandler
{
    int         iPort;
    Vector      vClientInfo;

    public ChatServer (int iPort)
    {
        this.iPort = iPort;
        this.vClientInfo = new Vector();

        this.start();
    }

    public void run ()
    {
        // Create new socket
        ServerSocket listener = null;
        try
        {
            listener = new ServerSocket(iPort);
        } catch (Exception e)
          {
            System.out.println("Error creating socket listener. "
                                    + e.toString());
            System.exit(1);
          }

        // listen for connections
        while (true)
        {
            try
            {
                System.out.println("Chat Server waiting for connections...");
                Socket newChatClient = listener.accept();

                // process new connection
                handleNewClient (newChatClient);
            } catch (IOException ioe)
              {
                System.out.println("Error handling new connection. "
                                    + ioe.toString());
                System.exit(1);
              }
        }

    }

    public void handleNewClient (Socket newClientSocket)
    {
        try
        {
            DataOutputStream dos =
                new DataOutputStream(newClientSocket.getOutputStream());
            DataInputStream dis =
                new DataInputStream(newClientSocket.getInputStream());


            ClientInfo newClient = new ClientInfo();

            // set Client Info parameters
            newClient.sUserName = dis.readUTF();    // get user name
            newClient.sockClient = newClientSocket;
            newClient.speakerChannel = dos;
            newClient.clientProxy = new ClientProxy(this, dis);

            System.out.println("Accepted connection from " +
                         "user: " + newClient.sUserName);


            vClientInfo.addElement(newClient);

            handleMessage("SYS_SERVER", newClient.sUserName +
                                        " joining conversation.");
        } catch (IOException ioe)
          {
            System.out.println("Error handling new client. "
                                   + ioe.toString());
          }
    }

    public synchronized void handleMessage (String sSpeakerName, String sMessage)
    {
        if (sSpeakerName.equals("DISCONNECT"))
        {
            // terminate client
            removeClient(sMessage);
        }
        else
        {
            // send to all clients
            int iNumClients = vClientInfo.size();
            for (int i = 0; i < iNumClients; i++)
            {
                ClientInfo clientInfo =
                    (ClientInfo) vClientInfo.elementAt(i);

                try
                {
                    clientInfo.speakerChannel.writeUTF(sSpeakerName);
                    clientInfo.speakerChannel.writeUTF(sMessage);
                } catch (IOException ioe)
                  {
                    System.out.println("Error sending message. " +
                                            ioe.toString());
                  }
            }
        }
    }

    private void removeClient(String sUserName)
    {
        int iNumClients = vClientInfo.size();
        for (int i = 0; i < iNumClients; i++)
        {
            ClientInfo clientInfo =
                    (ClientInfo) vClientInfo.elementAt(i);
            if (clientInfo.sUserName.equals(sUserName))
            {
                // clean up
                clientInfo.clientProxy.stopListen();
                try
                {
                    clientInfo.sockClient.close();
                } catch (IOException ioe) { }

                // remove from vector
                vClientInfo.removeElementAt(i);

                break;
            }
        }

        System.out.println("User " + sUserName + " has disconnected.");
        handleMessage("SYS_SERVER", sUserName +
                                    " has left conversation.");

    }

    public static void main (String args[])
    {
        if (args.length != 1)
        {
            System.out.println("Usage: java jwiley.chp9.ChatServer <port>");
            System.exit(1);
        }

        try
        {
            int iPort = Integer.parseInt(args[0]);
            new ChatServer(iPort);
        } catch (NumberFormatException ife)
          {
            System.out.println("Port must be an integer. " +
                                    ife.toString());
            System.exit(1);
          }
    }
}

class ClientInfo
{
    String              sUserName;
    Socket              sockClient;
    DataOutputStream    speakerChannel;
    ClientProxy         clientProxy;
}