
package jwiley.chp9;

import java.io.IOException;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.net.DatagramPacket;

public class DChatClient implements IChatClient, IMessageHandler
{
    IChatDisplay    display;
    DClientProxy    clientProxy;

    String          sUserName;
    InetAddress     serverAddress;
    int             serverPort;

    int             clientPort;

    public DChatClient (String sHostName, int iServerPort, int iClientPort)
    {
        try
        {
            this.serverAddress = InetAddress.getByName(sHostName);
            this.serverPort = iServerPort;
            this.clientPort = iClientPort;
        } catch (UnknownHostException uhe)
          {
            System.out.println("Error constructing ChatClient. "
                                    + uhe.toString());
          }
    }

    public void connect (String sUserName)
    {
        try
        {
            this.clientProxy = new DClientProxy(this, clientPort);
        } catch (IOException ioe)
          {
            display("Error connecting. Error: " + ioe.toString());
          }

        sendPacket("CONNECT:" + sUserName);

        this.sUserName = sUserName;
    }

    public void disconnect ()
    {
        sendPacket("DISCONNECT:" + sUserName);
        display("Bye.");

        clientProxy.stopListen();
        clientProxy = null;
    }

    public void setChatDisplay (IChatDisplay chatDisplay)
    {
        this.display = chatDisplay;
    }

    public void handleMessage (String sSpeaker, String sMessage)
    {
        if (sSpeaker.equals("SYS_SERVER"))
            display(">> " + sMessage);
        else
            display(sSpeaker + ": " + sMessage);
    }

    public void speak (String sMessage)
    {
        sendMessage(this.sUserName, sMessage);
    }

    private void sendMessage (String sSpeaker, String sMessage)
    {
        sendPacket(sSpeaker + ":" + sMessage);
    }

    private void display (String sMessage)
    {
        if (display == null)
            System.out.println(sMessage);
        else
            display.showMessage(sMessage);
    }

    private void sendPacket (String sMessage)
    {
        byte [] b = sMessage.getBytes();
        DatagramPacket p = new DatagramPacket(b, b.length,
                                              serverAddress,
                                              serverPort);

        clientProxy.sendPacket(p);
    }
}