
package jwiley.chp9;

import java.awt.*;
import java.awt.event.*;

public class DChatFrame extends Frame implements ActionListener
{
    Button  btnConnect = new Button("Connect");
    Button  btnDisconnect = new Button("Disconnect");
    Button  btnSend = new Button("Send");

    ChatDisplay display = new ChatDisplay();
    TextField   txtNewMessage = new TextField();
    TextField   txtUserName = new TextField(10);

    IChatClient chatClient;

    public DChatFrame (IChatClient chatClient)
    {
        this.chatClient = chatClient;
        if (chatClient != null)
            chatClient.setChatDisplay(display);

        Panel pnlUserName = new Panel();
        pnlUserName.setLayout(new FlowLayout());

        pnlUserName.add(new Label("User Name:"));
        pnlUserName.add(txtUserName);
        pnlUserName.add(btnConnect);
        pnlUserName.add(btnDisconnect);

        Panel pnlNewMessage = new Panel();
        pnlNewMessage.setLayout(new BorderLayout());

        pnlNewMessage.add("Center", txtNewMessage);
        pnlNewMessage.add("East", btnSend);

        btnConnect.addActionListener(this);
        btnDisconnect.addActionListener(this);
        btnSend.addActionListener(this);

        setLayout(new BorderLayout());
        add("North", pnlUserName);
        add("Center", display);
        add("South", pnlNewMessage);

        setTitle("Chat Client");
        setBackground(Color.lightGray);

        setSize(350,350);
        show();

    }

    public void actionPerformed (ActionEvent evt)
    {
        Object target = evt.getSource();

        if (target == btnConnect)
        {
            display.clearDisplay();
            if (txtUserName.getText().length() == 0)
                display.showMessage("Please enter a user name");
            else
            {
                if (chatClient != null)
                    chatClient.connect(txtUserName.getText());
            }
        }
        else if (target == btnDisconnect)
        {
            if (chatClient != null)
                chatClient.disconnect();
        }
        else if (target == btnSend)
        {
            if (chatClient != null)
            {
                if (txtNewMessage.getText().length() > 0)
                {
                    chatClient.speak(txtNewMessage.getText());
                    txtNewMessage.setText("");
                    txtNewMessage.requestFocus();
                }
            }
        }
    }

    public static void main (String args[])
    {
        // get ip addr and port
        if (args.length != 3)
        {
            System.out.println("Usage: java jwiley.chp9.ChatFrame "
                             + "<chat server ip> <server port> <client port>");
            System.exit(1);
        }

        try
        {
            int iServerPort = Integer.parseInt(args[1]);
            int iClientPort = Integer.parseInt(args[2]);
            new DChatFrame(new DChatClient(args[0], iServerPort, iClientPort));
        } catch (NumberFormatException nfe)
          {
            System.out.println("Port must be an integer. " + nfe.toString());
            System.exit(1);
          }
    }

}