
package jwiley.chp9;

import java.io.IOException;

import java.net.DatagramSocket;
import java.net.DatagramPacket;
import java.net.InetAddress;

import java.util.Vector;
import java.util.StringTokenizer;
import java.util.NoSuchElementException;

public class DChatServer extends Thread implements IMessageHandler
{
    int             iPort;
    Vector          vClientInfo;
    DatagramSocket  listener;
    DatagramSocket  sender;

    static int      MSG_SIZE = 256;

    public DChatServer (int iPort)
    {
        this.iPort = iPort;
        this.vClientInfo = new Vector();

        this.start();
    }

    public void run ()
    {
        try
        {
            // Create new socket
            listener = new DatagramSocket(iPort);
            sender = listener;  // use same socket for sending and listening
        } catch (Exception e)
          {
            System.out.println("Error creating socket listener. "
                                    + e.toString());
            System.exit(1);
          }

        // listen for connections
        while (true)
        {
            try
            {
                System.out.println("Datagram Chat Server waiting for packets...");

                byte b[] = new byte[MSG_SIZE];
                DatagramPacket p = new DatagramPacket(b, b.length);
                listener.receive(p);

                // process packets
                handlePacket (p);
            } catch (IOException ioe)
              {
                System.out.println("Error handling packet. "
                                    + ioe.toString());
                System.exit(1);
              }
        }
    }

    public void handlePacket (DatagramPacket p)
    {
        // all packets of form: TYPE:Data
        // Example: CONNECT:Eric

        String sPacketData = new String(p.getData(), 0, p.getLength());
        StringTokenizer st = new StringTokenizer(sPacketData, ":");

        try
        {
            String sMessageType = st.nextToken();
            String sMessageData = st.nextToken();

            if (sMessageType.equals("CONNECT"))
            {
                handleNewClient(p, sMessageData);
            }
            else if (sMessageType.equals("DISCONNECT"))
            {
                removeClient(sMessageData);
            }
            else
            {
                // sMessageType is User Name
                handleMessage(sMessageType, sMessageData);
            }

        } catch (NoSuchElementException nsee)
          {
            System.out.println("Invalid message type from " + p.getAddress().toString());
          }
    }

    public void handleNewClient (DatagramPacket p, String sUserName)
    {
        DClientInfo newClient = new DClientInfo();

        // set Client Info parameters
        newClient.sUserName = sUserName;
        newClient.clientAddress = p.getAddress();
        newClient.clientPort = p.getPort();

        System.out.println("Accepted connection from " +
                             "user: " + newClient.sUserName);

        vClientInfo.addElement(newClient);

        handleMessage("SYS_SERVER", newClient.sUserName +
                                    " joining conversation.");
    }

    public void handleMessage (String sSpeakerName, String sMessage)
    {
        // send to all clients
        int iNumClients = vClientInfo.size();
        for (int i = 0; i < iNumClients; i++)
        {
            DClientInfo clientInfo =
                (DClientInfo) vClientInfo.elementAt(i);

            try
            {
                String sPacketData = sSpeakerName + ":" + sMessage;
                byte [] b = sPacketData.getBytes();

                DatagramPacket p = new DatagramPacket(b, b.length,
                                                clientInfo.clientAddress,
                                                clientInfo.clientPort);
                sender.send(p);

            } catch (IOException ioe)
              {
                System.out.println("Error sending message. " +
                                        ioe.toString());
              }
        }
    }

    private void removeClient(String sUserName)
    {
        int iNumClients = vClientInfo.size();
        for (int i = 0; i < iNumClients; i++)
        {
            DClientInfo clientInfo =
                    (DClientInfo) vClientInfo.elementAt(i);
            if (clientInfo.sUserName.equals(sUserName))
            {
                // remove from vector
                vClientInfo.removeElementAt(i);

                break;
            }
        }

        System.out.println("User " + sUserName + " has disconnected.");
        handleMessage("SYS_SERVER", sUserName +
                                    " has left conversation.");

    }

    public static void main (String args[])
    {
        if (args.length != 1)
        {
            System.out.println("Usage: java jwiley.chp9.DChatServer <port>");
            System.exit(1);
        }

        try
        {
            int iPort = Integer.parseInt(args[0]);
            new DChatServer(iPort);
        } catch (NumberFormatException ife)
          {
            System.out.println("Port must be an integer. " +
                                    ife.toString());
            System.exit(1);
          }
    }
}

class DClientInfo
{
    String              sUserName;
    InetAddress         clientAddress;
    int                 clientPort;
}